(*  This file contains functions for generating separations of arithmetic operations.
    version 1  11-09-99
    version 1a 20-09-99 - ValueTableExpand replaced TruthTableExpand. 
*)

BeginPackage["PrimaryMath`ArithmeticExercises`SeparatedArithmetic`"
, "PrimaryMath`Utilities`ValueTables`"
] ;

BottomRowNumbersVisible::usage=
"BottomRowNumbersVisible is an option of PartialMultiples which specifies whether the
numbers in the bottom row representing the sums of the columns are visible.\n
BottomRowNumbersVisible -> True  specifies that the numbers are visible.\n
BottomRowNumbersVisible -> False specifies that the numbers are omitted.  This
allows the students to put them in themselves."

CarriedStartNumberSeparationVisible::usage =
"CarriedStartNumberSeparationVisible is an option of SeparatedSubtraction which
specifies whether the separated parts of the start number in the carry 
part of the problem are visible.\n
CarriedStartNumberSeparationVisible -> True specifies that the numbers are visible.\n
CarriedStartNumberSeparationVisible -> False specifies that the numbers are omitted."

CarriedSubtractedNumberSeparationVisible::usage =
"CarriedSubtractedNumberSeparationVisible is an option of SeparatedSubtraction which
specifies whether the separated parts of the subtracted number in the carry
part of the problem are visible.\n
CarriedSubtractedNumberSeparationVisible -> True specifies that the numbers 
are visible.\n
CarriedSubtractedNumberSeparationVisible -> False specifies that 
the numbers are omitted."  

ColumnNumbersVisible::usage=
"ColumnNumbersVisible is an option of PartialMultiples which specifies whether the
numbers in the left hand column representing the splitting of the first number
into its digits times powers of ten are present.\n
ColumnNumbersVisible -> True  specifies that the numbers are present.\n
ColumnNumbersVisible -> False specifies that the numbers are omitted.  This
allows the students to put them in themselves."

FinalAnswerVisible::usage=
"FinalAnswerVisible is an option which specifies whether the final answer to
the arithmetic problem is given.\n
FinalAnswerVisible -> True specifies that the final answer is present.\n
FinalAnswerVisible -> False specifies that the final answer is omitted.  This
allows the students to put them in themselves."

IntermediateWorking::usage =
"IntermediateWorking is an option which specifies what type of intermediate working 
is to be shown."

IntermediateWorkingVisible::usage =
"IntermediateWorkingVisible is an option which specifies whether the intermediate
working is to be made visible."

LeftColumnNumbersVisible::usage =
"LeftColumnNumbersVisible is an option of PartialMultiples which specifies whether
the numbers in left column representing the splitting of the first number
into its digits times powers of ten are present.\n
LeftColumnNumbersVisible -> True  specifies that the numbers are present.\n
LeftColumnNumbersVisible -> False specifies that the numbers are omitted.  This
allows the students to put them in themselves."

PartialMultiples::usage=
"PartialMultiples[number1_Integer,number2_Integer,opts___Rule] creates a view of 
how the multiplication
of two numbers is broken down into multiplications between the digits in the numbers.  
This
emphasises the significance of digit placement and that each digit represents a number
multiplied by some power of ten." 

Options[PartialMultiples] =
  { LeftColumnNumbersVisible -> True
  , RightColumnNumbersVisible -> True 
  , PartialMultiplesVisible -> True 
  , TopRowNumbersVisible -> True
  , BottomRowNumbersVisible -> True  
  , FinalAnswerVisible -> True } ;

PartialMultiplesVisible::usage=
"PartialMultiplesVisible is an option of PartialMultiples which specifies whether
the partial multiples are present in the grid of numbers\n
PartialMultiplesVisible -> True   specifies that the partial multiples are present.\n
PartialMultiplesVisible -> False  specifies that the partial multiples are to be omitted.
This allows the students to put them in themselves."

RightColumnNumbersVisible::usage=
"RightColumnNumbersVisible is an option of PartialMultiples which specifies whether the
numbers in the right hand column representing the sums of the rows are visible.\n
RightColumnNumbersVisible -> True  specifies that the numbers are visible.\n
RightColumnNumbersVisible -> False specifies that the numbers are omitted.  This
allows the students to put them in themselves."

SeparatedNumbersVisible::usage=
"SeparatedNumbersVisible is an option which specifies whether the separated numbers
are visible.\n
SeparatedNumbersVisible -> True specifies that the separated numbers are visible.\n
SeparatedNumbersVisible -> False specifies that the separated numbers are omitted.  
This allows the students to put them in themselves."

TopRowNumbersVisible::usage=
"TopRowNumbersVisible is an option of PartialMultiples which specifies whether the
numbers in the top row representing the splitting of the second number
into its digits times powers of ten are present.\n
RowNumbersVisible -> True  specifies that the numbers are present.\n
RowNumbersVisible -> False specifies that the numbers are omitted.  This
allows the students to put them in themselves."

SeparatedAddition::usage=
"SeparatedAddition[integer__Integer?Positive] creates a view of how a sum is 
separated into parts, each part representing a column in the numbers being 
added together."

Options[SeparatedAddition] =
  { FinalAnswerVisible -> True , SeparatedNumbersVisible -> True } ;

SeparatedSubtraction::usage=
"SeparatedSubstraction[startNumber_Integer?Positive,subtraction_Integer?Positive] 
creates a view of how a subtraction is separated into
parts, each part representing a column in the numbers being subtracted."

Options[SeparatedSubtraction] =
  { FinalAnswerVisible -> True , CarriedStartNumberSeparationVisible->True
  , StartNumberSeparationVisible -> True 
  , CarriedSubtractedNumberSeparationVisible->True
  , SubtractedNumberSeparationVisible -> True 
  } ;

StartNumberSeparationVisible::usage =
"StartNumberSeparationVisible is an option of SeparatedSubtraction which specifies
whether the start number's separation into its digits times powers of ten is present.\n
StartNumberSeparationVisible -> True  specifies that the numbers are present.\n
StartNumberSeparationVisible -> False  specifies that the numbers are omitted.  This
allows the students to put them in themselves."

SubtractedNumberSeparationVisible::usage =
"SubtractedNumberSeparationVisible is an option of SeparatedSubtraction which specifies
whether the subtracted number's separation into its digits times powers of ten is 
present.\n
SubtractedNumberSeparationVisible -> True  specifies that the numbers are present.\n
SubtractedNumberSeparationVisible -> False specifies that the numbes are omitted.  This
allows the students to put them in themselves."

VerticalAddition::usage =
"VerticalAddition[number1_Integer,number2_Integer,numbers___Integer,opts___Rule] 
creates a view of the addition of number1, number2, and any additional numbers.\n
Its option, IntermediateWorking, has the possible values, None (no intermediate
workings) MostSignificantDigitsFirst (the most significant digits are added first)
and LeastSignificantDigitsFirst (the least significant digits are added first)."

MostSignificantDigitsFirst::usage=
"MostSignificantDigitsFirst is a value of options such as IntermediateWorking that
signifies that the most significant digits are to be displayed first."

LeastSignificantDigitsFirst::usage=
"LeastSignificantDigitsFirst is a value of options such as IntermediateWorking that
signifies that the least significant digits are to be displayed first."

Options[VerticalAddition] =
  { FinalAnswerVisible -> True 
  , IntermediateWorking -> MostSignificantDigitsFirst 
  , IntermediateWorkingVisible -> True 
  } ;



Begin["`Private`"] ;

PartialMultiples[number1_Integer,number2_Integer,opts___Rule]:=
  Module[ {answer , bottomNumbersVisible , bottomRow, columnList, leftNumbersVisible 
    , finalAnswerPresent , grid , i , integerDigits , j 
    , leftColumn , multiplesVisible , rightColumn , rightNumbersVisible 
    , rowList , topRowNumbersVisible , topRow
    }
  ,
    (* Read in the options *)
    { bottomNumbersVisible , leftNumbersVisible , finalAnswerPresent 
    , multiplesVisible , rightNumbersVisible, topRowNumbersVisible } =
      { BottomRowNumbersVisible 
      , LeftColumnNumbersVisible , FinalAnswerVisible , PartialMultiplesVisible 
      , RightColumnNumbersVisible
      , TopRowNumbersVisible } /. { opts } /. Options[PartialMultiples] ;
    integerDigits = IntegerDigits[number1] ;
    columnList =
      Table[ integerDigits[[i]] * 10^(Length[integerDigits]-i)
      , { i , Length[integerDigits] }
      ] ;
    integerDigits = IntegerDigits[number2] ;
    rowList = 
      Table[ integerDigits[[i]] * 10^(Length[integerDigits]-i)
      , { i , Length[integerDigits] }
      ] ;
    multiplesVisible = 
      ValueTableExpand[ multiplesVisible , Length[columnList], Length[rowList] ] ;
    multiples =
      Table[ VisibleString[ rowList[[i]] * columnList[[j]] , multiplesVisible[[j,i]] ]
      , { j , Length[columnList] } 
      , { i , Length[rowList] }
      ] ;
    leftNumbersVisible = 
      ValueTableExpand[ leftNumbersVisible , Length[columnList] ] ;
    leftColumn = VisibleString[ columnList , leftNumbersVisible ] ;
    rightNumbersVisible = 
      ValueTableExpand[ rightNumbersVisible , Length[columnList] ] ;
    rightColumn = 
      Table[ VisibleString[columnList[[i]] * number2 ,rightNumbersVisible[[i]] ]
      , {i,Length[columnList]}] ;
    topRowNumbersVisible = ValueTableExpand[topRowNumbersVisible,Length[rowList]] ;
    topRow =
      Join[ { " " } 
      , Table[ VisibleString[ rowList[[i]] , topRowNumbersVisible[[i]] ] 
        , {i,Length[rowList]} ]
      , {"sub-total"}
      ] ; 
    bottomNumbersVisible =
      ValueTableExpand[bottomNumbersVisible , Length[rowList] ] ;
    bottomRow =
      Join[ {"sub-total"}
      , Table[ VisibleString[rowList[[i]] * number1 , bottomNumbersVisible[[i]] ]
        , {i,Length[rowList]} 
        ]
      , {VisibleString[ number1 * number2 , finalAnswerPresent ]}
      ] ;
    grid =
      GridBox[
        Join[ { topRow }
        , Transpose[
            Join[ { leftColumn } , Transpose[ multiples ] , {rightColumn} ]
          ]
        , { bottomRow }
        ]
      , ColumnLines->Join[{True},Table[False,{Length[rowList]-1}],{True}] 
      , RowLines->Join[{True},Table[False,{Length[columnList]-1}],{True}]
      ] ;
    answer = VisibleString[ number1 * number2 , finalAnswerPresent ] ;
    DisplayForm[
      RowBox[
        { ToString[number1]
        , "\[Times]"
        , ToString[number2]
        , "= "
        , grid
        , " ="
        , answer
        }
      ]
    ]
  ] ;

SeparatedAddition[integers__?Positive,opts___Rule] :=
  Module[ { additionLength , finalAnswerPresent 
      , i , integerDigits , integerList = List[integers] 
      , j , separatedNumbers , separatedNumberTerms , separatedNumbersVisible
      , startSum
      }
  ,  
    { finalAnswerPresent , separatedNumbersVisible } =
      { FinalAnswerVisible , SeparatedNumbersVisible 
      } /. { opts } /. Options[SeparatedAddition] ;
    (* Generate the representation of the starting sum *)
    startSum = 
      RowBox[
        Drop[
          Flatten[
            Table[
              { "+" , ToString[integerList[[i]]] }
            , { i , Length[integerList] }
            ]
          ]
        , 1
        ]
      ] ;
    
    integerDigits =
      Map[ IntegerDigits , integerList ] ;
    (* Ensure that all the entries are the same length *)
    additionLength = 
      Max[ Map[Length,integerDigits] ] ;
    Do[
      If[ Length[integerDigits[[i]] ] < additionLength
      , integerDigits[[i]] = 
          Join[ 
            Table[0,{additionLength-Length[integerDigits[[i]]]}]
          , integerDigits[[i]]
          ] 
      ]
    , { i , Length[integerDigits] }
    ] ;
    (* Create the separated number terms *)
    separatedNumberTerms =
      Table[ integerDigits[[i,j]] * 10^(Length[integerDigits[[i]]]-j)
      , { i , Length[integerDigits] }
      , { j , Length[integerDigits[[i]]] }
      ] ;
    separatedNumbersVisible = 
      ValueTableExpand[ separatedNumbersVisible 
      , Length[integerDigits] , Map[ Length,integerDigits ]
      ] ; 
    separatedNumbers =
      Table[
        Flatten[ 
          Table[ 
            { "+" 
            , VisibleString[separatedNumberTerms[[i,j]] 
              , separatedNumbersVisible[[i,j]]
              ] 
            }
          , {j, Length[separatedNumberTerms[[i]]]}
          ]
        ]
      , { i , Length[integerDigits] }
      ] ; 
    separatedNumbers[[1,1]] = "" ;
    answer = 
      VisibleString[ Apply[Plus,integerList] , finalAnswerPresent ] ;
    grid = 
      GridBox[
        separatedNumbers    
      , ColumnAlignments -> {Right}
      , ColumnLines -> False
      , RowLines -> False
      ] ;
    (* Display the computation *)
    DisplayForm[
      RowBox[
        { startSum
        , "= "
        , grid
        , " ="
        , answer
        }
      ]
    ]

  ] ;

SeparatedSubtraction[startNumber_Integer?Positive,subtractedNumber_Integer?Positive
, opts___Rule]:=
  Module[ { carriedStartNumberVisible , carriedSubtractedNumberVisible
      , carryFlag = False 
      , finalAnswerVisible , grid , gridSimplification , i , integerDigits 
      , separatedStartNumber , separatedStartNumberTerms 
      , separatedSubtractedNumber , separatedSubtractedNumberTerms
      , startNumbersVisible
      , subtractedNumbersVisible 
      }
  , 
    { carriedStartNumberVisible , carriedSubtractedNumberVisible 
    , finalAnswerVisible 
    , startNumbersVisible 
    , subtractedNumbersVisible } =
      { CarriedStartNumberSeparationVisible , CarriedSubtractedNumberSeparationVisible
      , FinalAnswerVisible
      , StartNumberSeparationVisible 
      , SubtractedNumberSeparationVisible
      } /. { opts } /. Options[SeparatedSubtraction] ;
    integerDigits = IntegerDigits[startNumber] ;
    separatedStartNumberTerms =  
      Table[ integerDigits[[i]] * 10^(Length[integerDigits]-i)
      , { i , Length[integerDigits] }
      ] ;
    startNumbersVisible =
      ValueTableExpand[ startNumbersVisible 
      , Length[separatedStartNumberTerms] ] ;
    separatedStartNumber =
      Drop[ 
        Flatten[ 
          Table[ 
            { VisibleString[separatedStartNumberTerms[[i]] 
              , startNumbersVisible[[i]] 
              ] 
            , "+" }
          , {i, Length[separatedStartNumberTerms]}
          ]
        ]
      , -1 
      ] ;
    integerDigits = IntegerDigits[subtractedNumber] ;
    separatedSubtractedNumberTerms = 
      Table[ integerDigits[[i]] * 10^(Length[integerDigits]-i)
      , { i , Length[integerDigits] }
      ] ;
    If[ Length[separatedSubtractedNumberTerms] < Length[separatedStartNumberTerms]
    , separatedSubtractedNumberTerms =
        Join[
          Table[ 0 
          , {Length[separatedStartNumberTerms]-Length[separatedSubtractedNumberTerms]}
          ]
        , separatedSubtractedNumberTerms
        ] 
    ] ;
    subtractedNumbersVisible = 
      ValueTableExpand[ subtractedNumbersVisible 
      , Length[separatedStartNumberTerms]
      ] ;
    separatedSubtractedNumber =
      Drop[
        Flatten[
          Table[ 
            { VisibleString[separatedSubtractedNumberTerms[[i]] 
              , subtractedNumbersVisible[[i]] 
              ] 
            , "-" }
          , { i , Length[separatedSubtractedNumberTerms] }
          ]
        ]
      , -1 
      ] ;
    grid = 
      GridBox[
        { Prepend[ separatedStartNumber , " " ]
        , Prepend[ separatedSubtractedNumber , "-" ]
        }
      , ColumnAlignments -> {Right}    
      , ColumnLines -> False
      , RowLines -> False
      ] ;
    (* Test for number carrying, and carry out number carrying where required *)
    Do[
      If[ separatedSubtractedNumberTerms[[i]] > separatedStartNumberTerms[[i]]
      , separatedStartNumberTerms[[i]] += 10^(Length[separatedStartNumberTerms]-i+1) ;
        separatedSubtractedNumberTerms[[i-1]] += 
          10^(Length[separatedStartNumberTerms]-i+1) ;
        carryFlag = True 
      ]
    , { i , Length[separatedSubtractedNumberTerms] , 2 , -1 }
    ] ;
    If[ separatedSubtractedNumberTerms[[1]] > separatedStartNumberTerms[[1]]
    , separatedStartNumberTerms[[1]] += 10^(Length[separatedStartNumberTerms]) ;
      PrependTo[separatedStartNumberTerms,0] ;
      PrependTo[separatedSubtractedNumberTerms
      , 10^(Length[separatedSubtractedNumberTerms])
      ] ;
      carryFlag = True ;
    ] ;
    gridSimplification =
      If[ carryFlag 
      , carriedStartNumberVisible =
          ValueTableExpand[ carriedStartNumberVisible 
          , Length[ separatedStartNumberTerms] ] ;
        separatedStartNumber =
          Drop[ 
            Flatten[ 
              Table[ 
                { VisibleString[separatedStartNumberTerms[[i]] 
                  , carriedStartNumberVisible[[i]] 
                  ] 
                , "+" }
              , {i, Length[separatedStartNumberTerms]}
              ]
            ]
          , -1 
          ] ;
        carriedSubtractedNumberVisible =
          ValueTableExpand[ carriedSubtractedNumberVisible
          , Length[ separatedSubtractedNumberTerms] 
          ] ;
        separatedSubtractedNumber =
          Drop[
            Flatten[
              Table[ 
                { VisibleString[separatedSubtractedNumberTerms[[i]]
                  , carriedSubtractedNumberVisible[[i]]
                  ] 
                , "-" }
              , { i , Length[separatedSubtractedNumberTerms] }
              ]
            ]
          , -1 
          ] ;
        RowBox[{ " ="
        , GridBox[
            { Prepend[ separatedStartNumber , " " ]
            , Prepend[ separatedSubtractedNumber , "-" ]
            }  
          , ColumnAlignments -> Right  
          , ColumnLines -> False
          , RowLines -> False
          ]
        }]
      , "" 
      ] ;  
    answer = 
      VisibleString[ startNumber - subtractedNumber , finalAnswerVisible ] ;
    DisplayForm[
      RowBox[
        { ToString[startNumber]
        , "-"
        , ToString[subtractedNumber]
        , "= "
        , grid
        , gridSimplification
        , " ="
        , answer
        }
      ]
    ]
  ] ;



VerticalAddition[ number1_Integer, otherNumbers__Integer, opts___Rule ] :=
  Module[ { answer , finalAnswerVisible , digitsList , i , intermediateResults 
      , intermediateWorking , intermediateWorkingVisible 
      , maxDigits , numberList , rowLinePositions 
      , startingAdditionTable 
      }
  , { finalAnswerVisible , intermediateWorking , intermediateWorkingVisible } =
      { FinalAnswerVisible , IntermediateWorking , IntermediateWorkingVisible } /. 
        { opts } /. Options[VerticalAddition] ;
   numberList = List[ number1 , otherNumbers ] ;
   startingAdditionTable = 
     Table[ { " " , ToString[numberList[[i]]] } , {i,Length[numberList]} ] ;
   startingAdditionTable[[Length[numberList],1]] = "+" ;
  

   If[ intermediateWorking =!= None
   , digitsList = Map[IntegerDigits, numberList ]  ;
     maxDigits = Max[ Map[ Length , digitsList ]] ;
     Do[
       digitsList[[i]] = 
         Join[ Table[ 0 , {maxDigits-Length[ digitsList[[i]] ]}]
         , digitsList[[i]]
         ] ;
     , { i , Length[digitsList] }
     ] ;
  
     intermediateResults = 
       Table[
         (10^(maxDigits-i)) * 
         Apply[ Plus 
         , Table[digitsList[[j,i]] , { j , Length[digitsList] } ]
         ]
       , { i , maxDigits }
       ] ;
  
     If[ intermediateWorking === LeastSignificantDigitsFirst
     , intermediateResults = Reverse[intermediateResults]
     ] ;
     intermediateWorkingVisible =
       ValueTableExpand[ intermediateWorkingVisible , Length[intermediateResults] ] ;
   ] ;
  
   rowLinePositions =
     Join[ 
       Table[ False , {Length[numberList]-1} ]
     , { True }
     , Switch[ intermediateWorking
       , MostSignificantDigitsFirst | LeastSignificantDigitsFirst
         , Join[ Table[ False , {Length[intermediateResults]-1} ] , {True } ]
       , _
         , {} 
       ]
     , {False}
     ] ;
     
   answer = 
     VisibleString[ Apply[Plus,numberList] , finalAnswerVisible ] ;

    DisplayForm[
      GridBox[
        Join[
          startingAdditionTable
        , If[ intermediateWorking === None
          , { }
          , MapThread[ ({" " , VisibleString[#1,#2] })& 
            , {intermediateResults , intermediateWorkingVisible}
            ]
          ]
        , {{ " " , answer }}
        ]
      , ColumnAlignments -> {Right}
      , RowLines -> rowLinePositions
      ]
    ]
  ] ;


Protect[PartialMultiples,SeparatedAddition,SeparatedSubtraction,VerticalAddition] ;

End[] ;

EndPackage[] ;