(*  This file contains functions for creating and manipulating number squares. 
    version 1  06-07-99
*)

BeginPackage["PrimaryMath`EnclosedShapes`EnclosedShapes`"
, "Utilities`FilterOptions`"] ;

EnclosedShape::usage=
"EnclosedShape[shape_,opts___Rule] creates a new shape within the specified shape."

Options[EnclosedShape] = 
  { SideJump -> 1, EnclosedShapeState -> Automatic , EnclosedShapeVertexPosition -> 1/2 
  } ;

OtherOptions[EnclosedShape] =
  { EnclosedShapeState -> Filled , EnclosedShapeState -> OutLine } ;

Begin["`Private`"] ;

EnclosedShape[shape_,opts___Rule] :=
  Module[{extendedOldLinePoints,i,oldLinePoints,shapeHead,shapeState,vertexRatio}
  ,
    { vertexRatio, shapeState , sideJump } = 
      { EnclosedShapeVertexPosition , EnclosedShapeState , SideJump } /. 
        {opts} /. Options[EnclosedShape] ;
    (* Choose the header for the enclosed shape *)
    Switch[ shapeState 
    , Filled
      , shapeHead = Polygon
    , OutLine
      , shapeHead = Line
    , Automatic
      , If[ Head[shape] === Polygon , shapeHead = Line , shapeHead = Polygon ]
    ] ;
    (* Generate the points of the new shape *)
    (* Create a list of points, dropping the last point, which
       for a closed shaped should correspond to the first *)
    oldLinePoints = Drop[shape[[1]],-1] ;  
    (* Create a list of points from sufficient repetitions of oldLinePoints
       to allow the drawn sides to go completely around the sides of the 
       original shape, (i.e. LCM[Length[oldLinePoints],sideJump]) + return to
       the original place (i.e. + 2 on the end).  *) 
    extendedOldLinePoints = 
      Flatten[ 
        Table[
          oldLinePoints
        , { Ceiling[(LCM[Length[oldLinePoints],sideJump]+2)/Length[oldLinePoints]] }
        ]
      , 1 ] ;
    newLinePoints =
      Table[
          extendedOldLinePoints[[i]] * (1 - vertexRatio) 
        + extendedOldLinePoints[[i+1]] * vertexRatio
      , {i , 1 , 1+LCM[Length[oldLinePoints],sideJump] , sideJump }
      ] ;
    shapeHead[ newLinePoints ]   
  ] /; (Head[shape] === Polygon ) || ( Head[shape] === Line ) ;

End[] ;

EndPackage[] ;