(*  This file contains functions for handling digital roots.
    version 1.0  06-07-99
    version 1.1  05-09-99
*)

BeginPackage["PrimaryMath`NumberGames`DigitalRoots`DigitalRoots`"
, "Utilities`FilterOptions`"] ;

AngleOffset::usage =
"AngleOffset is an option of DigitalRootPattern which specifies the offset to be applied
to the numbers.\n
AngleOffset -> 0  specifies that the lowest number will be positioned at 12 o'clock.\n
AngleOffset -> 1  specifies that the lowest number will be positioned to the right
of the highest number, which is positioned at 12 o'clock."

Base::usage =
"Base is an option which sets the base to be used."

DigitalRoot::usage=
"DigitalRoot[integer_Integer] returns the digital root of the integer.
The digital root is generated by summing the integer's digits.  If this has more than
one digit, then the sum is applied again and again until the result does not change.\n
DigitalRoot[list_List] returns the digital root of a list of integers."

Options[DigitalRoot] = { Base -> 10 } ;

DigitalRootPattern::usage=
"DigitalRootPattern[] generates a digital root pattern which the student can draow 
digital root patterns on.\n
DigitalRootPattern[list_List], where list is a list of integers, draws the digital
root pattern for the list of integers."

Options[DigitalRootPattern] =
  { AngleOffset -> 0 , TextRadius -> 1.2 } ;

TextRadius::usage =
"TextRadius is an option of DigitRootPattern which specifies the radius that 
the numbers are positioned at." 


Begin["`Private`"] ;

DigitalRoot[list_List,opts___Rule ] :=
  Map[ (DigitalRoot[#,opts])& , list ] /; 
    VectorQ[list,(IntegerQ[#] && Positive[#] )& ] ;

DigitalRoot[integer_Integer?Positive,opts___Rule] := 
  Module[ {base } 
  , { base } = 
      { Base } /. {opts} /. Options[DigitalRoot] /. Options[DigitalRootPattern] ; 
  FixedPoint[ (Apply[Plus,IntegerDigits[#,base]])& , integer ] 
  ] ;
 
DigitalRootPattern[opts___Rule] :=
  Module[ { angleOffset , base , i , textRadius }
  , { angleOffset , base , textRadius } = 
      { AngleOffset , Base , TextRadius } /. {opts } /. Options[DigitalRoot] /. 
        Options[DigitalRootPattern] ;
    Show[
      Graphics[
        {
          Table[ 
            Text[ i
            ,   textRadius
              * { Sin[(i+angleOffset)*2Pi/(base-1)] , Cos[(i+angleOffset)*2Pi/(base-1)] }
            ]
          , { i , 1 , base-1 }
          ]
        }
      , FilterOptions[Graphics,opts], AspectRatio->1
      ]
    ]
  ] ;

DigitalRootPattern[list_List,opts___Rule] :=
  Module[ {  angleOffset  , template }
  , { angleOffset , base } = 
      { AngleOffset , Base } /. {opts} /. Options[DigitalRoot] /.
        Options[DigitalRootPattern] ; 
    Show[
      { DigitalRootPattern[DisplayFunction->Identity,opts] 
      , Graphics[
          {
            Line[
              Map[ 
               ({ Sin[(#+angleOffset)*2Pi/(base-1)], Cos[(#+angleOffset)*2Pi/(base-1)] })&
              , Map[ (DigitalRoot[#,opts])& , list ]
              ]
            ]
          }
        ]
      }
    , FilterOptions[Graphics,opts] 
    , AspectRatio->1 , DisplayFunction->$DisplayFunction
    ]
  ] /; VectorQ[list,(IntegerQ[#] && Positive[#])&] ;


End[] ;

EndPackage[] ;