(*  This file contains functions for creating and manipulating number grids. 
    version 1  26-02-00
*)

BeginPackage["PrimaryMath`NumberGrids`NumberGrids`"
, "PrimaryMath`Utilities`StyleTables`" 
, "PrimaryMath`Common`NumberOptions`"
, "Utilities`FilterOptions`"] ;

BottomToTop::usage =
"BottomToTop is a value of NumberGridRowOrder that specifies that the
rows start at the bottom of the grid, and run to the top."

LeftToRight::usage =
"LeftToRight is a value of NumberGridColumnOrder that specifies that the
columns start on the left of the number grid, and run to the right."

NumberGrid::usage=
"NumberGrid[opts___Rule] creates a number grid with properties determined
by its options."

NumberGridColumnOrder::usage =
"NumberGridColumnOrder is an option of NumberGrid that specifies the order
of the columns.\n
NumberGridColumnOrder->LeftToRight specifies that the columns go from left
to right.\n
NumberGridColumnOrder->RightToLeft specifies that the columns go from right
to left."

NumberGridEndNumber::usage = 
"NumberGridEndNumber is an option of NumberGrid which sets the last number in 
the grid."

NumberGridRowOrder::usage =
"NumberGridRowOrder is an option of NumberGrid that specifies the order
of the rows.\n
NumberGridRowOrder->TopToBottom specifies that the rows go from top 
to bottom.\n
NumberGridRowOrder->BottomToTop specifies that the rows run from
bottom to top."

NumberGridStartNumber::usage =
"NumberGridStartNumber is an option of NumberGrid which sets the starting number
in the grid."

NumberGridWidth::usage =
"NumberGridWidth is an option of NumberGrid which sets the width of the rows of the
grid.\n
e.g. NumberGridWidth->10 sets the width to 10."

RightToLeft::usage =
"RightToLeft is a value of NumberGridColumnOrder that specifies that the
columns start on the right of the number grid and run to the left."

TopToBottom::usage =
"TopToBottom is a value of NumberGridRowOrder that specifies that the rows start
at the top of the grid, and run to the bottom."

Options[NumberGrid] = 
  { NumberGridColumnOrder -> LeftToRight
  , NumberGridStartNumber ->1 , NumberGridEndNumber -> 100
  , NumberGridRowOrder -> TopToBottom 
  , NumberGridWidth -> 10 
  , NumberStyles -> {}
  } ;

Begin["`Private`"] ;

NumberGrid[opts___Rule] :=
  Module[{ columnOrder,endNumber,gridLength, hsign, i
    , num, numList, numOption, numberStyles, rowOrder, startNumber
    , vsign, width}
  ,
    { columnOrder, endNumber, rowOrder, startNumber, width 
    , numberStyles } = 
      { NumberGridColumnOrder, NumberGridEndNumber
      , NumberGridRowOrder, NumberGridStartNumber , NumberGridWidth 
      , NumberStyles
      } /. {opts} /. Options[NumberGrid] ;
    (* Set up the values used to determine how the grid runs *)
    hsign = If[ columnOrder === RightToLeft , -1 , 1 ] ;
    vsign = If[ rowOrder === BottomToTop , 1 , -1 ] ;
    gridLength = endNumber - startNumber + 1 ;
    numberOfColumns = Ceiling[ gridLength / width ] ;
    (* Set the default value of the text options *)
    Do[ numOption[i] = {} 
    , { i , startNumber , endNumber } ] ;
    (* Process the number styles definition *)
    { numList , numberStyles} = 
      StyleTableExpand[numberStyles,StyleTableOrdering->Append] ;
    Do[ 
      numOption[numberStyles[[i,1]] ] = 
        {FilterOptions[Text,numberStyles[[i,2]]]} 
    , { i , Length[numberStyles] }
    ] ; 
    (* Draw the number grid *)
    Show[
      Graphics[
        Table[
          num = i+startNumber-1 ;
          Apply[ Text
          , Join[
              { num 
              , { hsign*(i-width*Floor[(i-1)/width])
                , vsign*(Floor[(i-1)/width]+1)}
              , {0,0}
              }
            , numOption[num]
            ]
          ]
        , {i,1,endNumber-startNumber+1}
        ]
      ]
    , FilterOptions[Graphics,opts]
    , AspectRatio->numberOfColumns/width
    , PlotRange->{hsign*{0.5,width+0.5},vsign*{0.5,numberOfColumns+0.5}} 
    ]
  ];

End[] ;

EndPackage[] ;