(*  This file contains functions for creating pictures formed by
    stretching strings between lines. 
    version 1  06-07-99
*)

BeginPackage["PrimaryMath`StringShapes`StringShapes`"] ;

StringShape::usage=
"StringShape[line1_Line,line2_Line] creates a picture of lines stretched at equal 
intervals between line1 and line2."

Options[StringShape] = { NumberOfStretchedLines -> 10 } ;

Begin["`Private`"] ;

LineLengths[linePoints_List] :=
  Module[ {i , runningLength = 0 } 
  , Prepend[ 
      Table[
        runningLength +=
          Sqrt[
            Apply[Plus
            , Map[ (#^2)& , linePoints[[i]] - linePoints[[i-1]] ]
            ]
          ]
      , { i , 2 , Length[linePoints] }
      ]
    , 0 ]
  ] ;

LinePosition[ distance_?NumberQ , lineLengths_List , linePositions_List ] :=
  Module[ { diff  , i } 
  , For[ i=1
    , lineLengths[[i]] < distance
    , i++
    ] ;
    diff = 
      (distance - lineLengths[[i-1]])/(lineLengths[[i]]-lineLengths[[i-1]]) ;
    (1-diff)* linePositions[[i-1]] + diff * linePositions[[i]]
  ] ;

StringShape[line1_Line,line2_Line,opts___Rule] :=
  Module[{i,numberOfLines, lineLengths , lineVertices , offSet
    , stepSize , totalLineLength }
  ,
    { numberOfLines } = 
      { NumberOfStretchedLines } /. {opts} /. Options[StringShape] ;
    (* Calculate the length of the lines *)
    lineVertices[1] = line1[[1]] ;
    lineVertices[2] = line2[[1]] ;
    Do[ lineLengths[i] = LineLengths[ lineVertices[i] ] , {i,1,2} ];
    (* Create the plot *)
    Do[ 
      totalLineLength[i] = Last[lineLengths[i]] ;
      stepSize[i] = totalLineLength[i]/numberOfLines ;
      offSet[i] = -0.5 * (totalLineLength[i]/numberOfLines)
    , {i,1,2} ] ;
    
    Table[
      Line[
        { LinePosition[ i*stepSize[1] + offSet[1]
          , lineLengths[1] , lineVertices[1] ]
        , LinePosition[ i*stepSize[2] + offSet[2]
          , lineLengths[2] , lineVertices[2] ]
        }
      ]
    , {i,numberOfLines }
    ]
  ] ;

End[] ;

EndPackage[] ;