(*  This file contains functions for rearranging lists of objects in
    random order.   It is used by other packages.
    version 1  27-09-99
*)

BeginPackage["PrimaryMath`Utilities`RandomArrangements`"] ;

RandomArrangement::usage =
"RandomArrangement[l_List] rearranges the elements of l and returns
the answer."

RandomCut::usage = 
"RandomCut[l_List,n_Integer?Positive] cuts the list l n times in random
places.  Each time the list is cut, a random length central part of the
list is removed and placed at the end of the remainder of the list."


Begin["`Private`"] ;

RandomArrangement[l_List] :=
  Module[ { ansList = l , choice , i , tempList = l }
  , 
    Do[ 
      choice = Random[Integer,{1,i}] ; 
      ansList[[i]] = tempList[[choice]] ;
      Do[
        tempList[[j]] = tempList[[j+1]]
      , { j , choice , i-1 }
      ] ;
      , { i , Length[l] , 2 , -1 }
    ] ;
    ansList[[1]] = tempList[[1]] ;
    ansList
  ] ;

RandomCut[l_List,n_Integer?Positive] :=
  Module[ { cutStart , cutEnd , tempList , len = Length[l] } 
  , Nest[ 
      ( cutStart = Random[Integer,{1,len}] ;
        cutEnd = Random[Integer,{cutStart,len}] ;
        Join[Take[#,cutStart-1] , Take[#,{cutEnd+1,len}] , Take[#,{cutStart,cutEnd}]
        ]
      )&
    , l
    , n
    ] 
  ] ;

End[] ;

EndPackage[] ;