(*  This file contains functions for expanding options.  It is used by other
    packages.
    version 1  18-09-99
*)

BeginPackage["PrimaryMath`Utilities`ValueTables`"] ;

CycleEntries::usage =
"CycleEntries is a value of RepetitionMethod which indicates that the
table is to be expanded by repeatedly cycling through the original entries."

ValueTableExpand::usage =
"ValueTableExpand is a function which expands value tables.\n
As it is only used by other functions, the various ways in which 
it can be called are not explained here."

Options[ValueTableExpand] =
  { RepetitionMethod -> RepeatLastEntry } ;

RepeatLastEntry::usage =
"RepeatLastEntry is an value of RepetitionMethod which indicates that the table
is to be expanded by repeatedly adding the last specified entry."

RepetitionMethod::usage =
"RepetitionMethod is an option of ValueTableExpand.\n
RepetitionMethod -> RepeatLastEntry indicates that the table is to be
expanded by repeating the last entry.\n
RepetitionMethod -> CycleEntries indictates that the table is to be
expanded by cycling around the entries specified."

VisibleString::usage =
"VisibleString[ object_ , True] returns String[object].\n
VisibleString[ object_ , False] returns a string of spaces of the same length
as the characters in the string representation of the object."

Begin["`Private`"] ;

ValueTableExpand[ valueList_List , length_Integer , opts___Rule ] :=
  Module[ { repetitionMethod , vList }
  , If[ MemberQ[{CycleEntries,RepeatLastEntry},Last[valueList] ]
    , repetitionMethod = Last[valueList] ;
      vList = Drop[valueList,-1] ;
    , { repetitionMethod } = 
        { RepetitionMethod } /. {opts} /. Options[ValueTableExpand] ;
      vList = valueList 
    ] ;
    Which[
      Length[vList] < length
      , If[ repetitionMethod === CycleEntries
        , Take[
            Apply[Join,Table[ vList , { Ceiling[ length/Length[vList] ] } ] ]
          , length
          ] 
        , Join[ vList , Table[ Last[vList] , {length-Length[vList]} ] ]
        ]
    , Length[vList] > length
      , Take[ vList , length ]
    , True
      , vList
    ] 
  ] ;

ValueTableExpand[ valueList_List , length_Integer , width_Integer , opts___Rule ] :=
  ValueTableExpand[ valueList , length , Table[width,{length}] , opts ] ;

ValueTableExpand[ valueList_List , length_Integer , widthList_List , opts___Rule ] :=
  Module[ { correctedList, i , repetitionMethod, temp, vList }
  , (* First get the length of the list correct *)
    If[ MemberQ[{CycleEntries,RepeatLastEntry},Last[valueList] ]
    , repetitionMethod = Last[valueList] ;
      vList = Drop[valueList,-1] ;
    , { repetitionMethod } = 
        { RepetitionMethod } /. {opts} /. Options[ValueTableExpand] ;
      vList = valueList 
    ] ;
    correctedList =
      Which[ 
        Length[ vList ] < length
        , If[ repetitionMethod === CycleEntries
          , Take[
              Apply[Join,Table[ vList , { Ceiling[ length/Length[vList] ] } ] ]
            , length
            ] 
          , Join[ vList , Table[ Last[vList] , {length-Length[vList]} ] ]
          ]
      , Length[vList] > length
        , Take[ vList , length ]
      , True
        , vList
      ] ;
    (* Now make sure that each of the subLists has the correct length *)
    Do[
      temp = correctedList[[i]] ;
      correctedList[[i]] =
        If[ Head[temp] =!= List
        , Table[ temp , {widthList[[i]]} ]
        , If[ MemberQ[{CycleEntries,RepeatLastEntry},Last[temp] ]
          , repetitionMethod = Last[temp] ;
            vList = Drop[temp,-1] ;
          , { repetitionMethod } = 
              { RepetitionMethod } /. {opts} /. Options[ValueTableExpand] ;
            vList = temp 
          ] ;
          Which[
            Length[vList] < widthList[[i]] 
            , If[ repetitionMethod === CycleEntries
              , Take[
                  Apply[Join
                  , Table[ vList , { Ceiling[ widthList[[i]]/Length[vList] ] } ] ]
                , widthList[[i]]
                ] 
              , Join[ vList , Table[ Last[vList] , {widthList[[i]]-Length[vList]} ] ]
              ]
          , Length[vList] > widthList[[i]]
            , Take[vList , widthList[[i]] ]
          , True
            , vList
          ] 
        ] ;
    , { i , length }
    ] ;
    correctedList
  ] ;

ValueTableExpand[ value_ , length_Integer , opts___Rule] :=
  Table[ value , {length} ] /; (Head[value] =!= List);

ValueTableExpand[ value_ , length_Integer , width_Integer , opts___Rule ]:=
  Table[ value , {length},{width} ] /; (Head[value] =!= List);

ValueTableExpand[ value_ , length_Integer , widthList_List , opts___Rule ]:=
  Table[ value 
  , {i,length} , {widthList[[i]]} 
  ] /; (Head[value] =!= List) ;



VisibleString[ int_ , True ] := ToString[int] ;

VisibleString[ int_ , False ] := 
  StringJoin[ Table[ " " , {StringLength[ToString[int]]} ] ] ;

VisibleString[ int_List , truthValues_List ] :=
  MapThread[ VisibleString , {int,truthValues} ] ;


End[] ;

EndPackage[] ;