(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     16116,        406]*)
(*NotebookOutlinePosition[     16833,        431]*)
(*  CellTagsIndexPosition[     16789,        427]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Patterns formed by concentric circles", "Title"],

Cell["This notebook's file is called ConcentricCircles.nb", "SmallText"],

Cell[TextData[StyleBox[
"N.B.  This notebook was originally supplied with the graphics stripped out \
to save space.  To regenerate the graphics, evaluate the entire notebook.",
  FontColor->RGBColor[1, 0, 0]]], "SmallText"],

Cell[CellGroupData[{

Cell["Initialisation", "Section"],

Cell["\<\
Before running the examples in the rest of the notebook, execute the cells in \
the initialisation section!\
\>", "Text"],

Cell["\<\
The following line loads in a package which gives colour names in English\
\>", "Text"],

Cell[BoxData[
    \(\(Needs["\<Graphics`Colors`\>"]\ ; \)\)], "Input"],

Cell[CellGroupData[{

Cell["Handy Formulae", "Subsection"],

Cell["\<\
The foci are assumed to lie on the x axis, at equal distances to either side \
of the origin.\
\>", "Text"],

Cell[BoxData[
    \(\(Off[General::spell1]\ ; \)\)], "Input"],

Cell[CellGroupData[{

Cell["\<\
Formulae for ellipse in terms of foci separation, sum of lengths, and \
distance from left focus\
\>", "Subsubsection"],

Cell["\<\
Here are the formulae for the coordinates (relative to the left focus) of the \
point on the ellipse in terms of the foci separation, distance from left foci \
and distance between focii\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\({lowerSolution, upperSolution} = 
      Solve[{ypos\^2 == leftRadius\^2 - \((xpos + fociSeparation/2)\)\^2, 
          ypos\^2 == 
            \((lengthSum - leftRadius)\)\^2 - 
              \((fociSeparation/2 - xpos)\)\^2}, {xpos, ypos}]\ \)\)], "Input"],

Cell[BoxData[
    \({{ypos \[Rule] 
          \(-\(1\/2\)\)\ 
            \[Sqrt]\((
                \(-fociSeparation\^2\) + 4\ leftRadius\^2 - 
                  4\ leftRadius\ lengthSum + 2\ lengthSum\^2 - 
                  \(4\ leftRadius\^2\ lengthSum\^2\)\/fociSeparation\^2 + 
                  \(4\ leftRadius\ lengthSum\^3\)\/fociSeparation\^2 - 
                  lengthSum\^4\/fociSeparation\^2)\), 
        xpos \[Rule] 
          \(\((2\ leftRadius - lengthSum)\)\ lengthSum\)\/\(2\ fociSeparation
              \)}, {ypos \[Rule] 
          1\/2\ \[Sqrt]\((
                \(-fociSeparation\^2\) + 4\ leftRadius\^2 - 
                  4\ leftRadius\ lengthSum + 2\ lengthSum\^2 - 
                  \(4\ leftRadius\^2\ lengthSum\^2\)\/fociSeparation\^2 + 
                  \(4\ leftRadius\ lengthSum\^3\)\/fociSeparation\^2 - 
                  lengthSum\^4\/fociSeparation\^2)\), 
        xpos \[Rule] 
          \(\((2\ leftRadius - lengthSum)\)\ lengthSum\)\/\(2\ fociSeparation
              \)}}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[
"Formulae for ellipse in terms of foci separation, sum of lengths and angle \
\[Alpha] away from x-axis"], "Subsubsection"],

Cell[TextData[
"Here are the formulae for the coordinates (relative to the left focus) of \
the point on the ellipse in terms of the foci separation, distance between \
foci and angle \[Alpha] from the xaxis.   "], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\({solution[1], solution[2]} = 
      Solve[{ypos\^2 == leftRadius\^2 - \((xpos\  + \ fociSeparation/2)\)\^2, 
          ypos\^2 == 
            \((lengthSum - leftRadius)\)\^2 - 
              \((fociSeparation/2 - xpos)\)\^2, 
          ypos\  == \ xpos\ Tan[\[Alpha]]}, {xpos, ypos, leftRadius}]\ \ 
    \)\)], "Input"],

Cell[BoxData[
    \({{ypos \[Rule] 
          \(-\(\(lengthSum\ \@\(\(-fociSeparation\^2\) + lengthSum\^2\)\ 
                  Tan[\[Alpha]]\)\/\@\(\(-4\)\ fociSeparation\^2 + 
                    4\ lengthSum\^2 + 
                    4\ lengthSum\^2\ Tan[\[Alpha]]\^2\)\)\), 
        leftRadius \[Rule] 
          \(lengthSum\^2 - 
              \(2\ fociSeparation\ lengthSum\ 
                  \@\(\(-fociSeparation\^2\) + lengthSum\^2\)\)\/\@\(\(-4\)\ 
                      fociSeparation\^2 + 4\ lengthSum\^2 + 
                    4\ lengthSum\^2\ Tan[\[Alpha]]\^2\)\)\/\(2\ lengthSum\), 
        xpos \[Rule] 
          \(-\(\(lengthSum\ 
                  \@\(\(-fociSeparation\^2\) + lengthSum\^2\)\)\/\@\(\(-4\)\ 
                      fociSeparation\^2 + 4\ lengthSum\^2 + 
                    4\ lengthSum\^2\ Tan[\[Alpha]]\^2\)\)\)}, {
        ypos \[Rule] 
          \(lengthSum\ \@\(\(-fociSeparation\^2\) + lengthSum\^2\)\ 
              Tan[\[Alpha]]\)\/\@\(\(-4\)\ fociSeparation\^2 + 
                4\ lengthSum\^2 + 4\ lengthSum\^2\ Tan[\[Alpha]]\^2\), 
        leftRadius \[Rule] 
          \(lengthSum\^2 + 
              \(2\ fociSeparation\ lengthSum\ 
                  \@\(\(-fociSeparation\^2\) + lengthSum\^2\)\)\/\@\(\(-4\)\ 
                      fociSeparation\^2 + 4\ lengthSum\^2 + 
                    4\ lengthSum\^2\ Tan[\[Alpha]]\^2\)\)\/\(2\ lengthSum\), 
        xpos \[Rule] 
          \(lengthSum\ 
              \@\(\(-fociSeparation\^2\) + lengthSum\^2\)\)\/\@\(\(-4\)\ 
                  fociSeparation\^2 + 4\ lengthSum\^2 + 
                4\ lengthSum\^2\ Tan[\[Alpha]]\^2\)}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(On[General::spell1]\ ; \)\)], "Input"],

Cell[TextData[{
  "Analysis of these two solutions shows that solution[2] is valid in the \
first quadrant, and solution[1] is valid in the 2nd quadrant.  solution[1] is \
valid in the 3rd quadrant, and solution[2] is valid in the 4th quadrant.  \n\
The solutions are singular for \[Alpha] ",
  Cell[BoxData[
      \(TraditionalForm\`\( \[Rule] \(\[PlusMinus]\[Pi]\)\)\)]],
  "/2.  But here we know that xpos\[Rule]0, ",
  Cell[BoxData[
      \(TraditionalForm
      \`ypos \[Rule] 
        \(\[PlusMinus]\@\(lengthSum\^2 - fociSeparation\^2\)\)/2\)]],
  ", where the sign chosen is the same as that of \[Alpha].\nPutting all \
these observations together gives the general formula for xpos and ypos, \
given lengthSum, fociSeparation and \[Alpha]:"
}], "Text"],

Cell[BoxData[
    \(\(alphaSolution[fociSeparation_, lengthSum_, \[Alpha]_?NumberQ] := \n\t
      Module[{xpos, ypos}\n\t\t\t, 
        \ {xpos, ypos} /. 
          Which[\ \[Alpha]\  == \ \[Pi]/2\ , 
            \ {xpos\  \[Rule] 0, 
              ypos \[Rule] \@\(lengthSum\^2 - fociSeparation\^2\)/2}, 
            \[Alpha]\  == 3  \[Pi]/2, {xpos \[Rule] 0, 
              ypos \[Rule] \(-\@\(lengthSum\^2 - fociSeparation\^2\)\)/2}, 
            0 <= \[Alpha] < \[Pi]/2, {
              xpos \[Rule] 
                \(lengthSum\ 
                    \@\(\(-fociSeparation\^2\) + 
                        lengthSum\^2\)\)\/\@\(\(-4\)\ fociSeparation\^2 + 
                      4\ lengthSum\^2 + 4\ lengthSum\^2\ Tan[\[Alpha]]\^2\), 
              ypos \[Rule] 
                \(lengthSum\ \@\(\(-fociSeparation\^2\) + lengthSum\^2\)\ 
                    Tan[\[Alpha]]\)\/\@\(\(-4\)\ fociSeparation\^2 + 
                      4\ lengthSum\^2 + 4\ lengthSum\^2\ Tan[\[Alpha]]\^2\)}, 
            \[Pi]/2 < \[Alpha] <= \[Pi], {
              xpos \[Rule] 
                \(-\(\(lengthSum\ 
                        \@\(\(-fociSeparation\^2\) + 
                            lengthSum\^2\)\)\/\@\(\(-4\)\ fociSeparation\^2 + 
                          4\ lengthSum\^2 + 
                          4\ lengthSum\^2\ Tan[\[Alpha]]\^2\)\)\), 
              ypos \[Rule] 
                \(-\(\(lengthSum\ 
                        \@\(\(-fociSeparation\^2\) + lengthSum\^2\)\ 
                        Tan[\[Alpha]]\)\/\@\(\(-4\)\ fociSeparation\^2 + 
                          4\ lengthSum\^2 + 
                          4\ lengthSum\^2\ Tan[\[Alpha]]\^2\)\)\)}, 
            \[Pi] < \[Alpha] < 3  \[Pi]/2, {
              xpos \[Rule] 
                \(-\(\(lengthSum\ 
                        \@\(\(-fociSeparation\^2\) + 
                            lengthSum\^2\)\)\/\@\(\(-4\)\ fociSeparation\^2 + 
                          4\ lengthSum\^2 + 
                          4\ lengthSum\^2\ Tan[\[Alpha]]\^2\)\)\), 
              ypos \[Rule] 
                \(-\(\(lengthSum\ 
                        \@\(\(-fociSeparation\^2\) + lengthSum\^2\)\ 
                        Tan[\[Alpha]]\)\/\@\(\(-4\)\ fociSeparation\^2 + 
                          4\ lengthSum\^2 + 
                          4\ lengthSum\^2\ Tan[\[Alpha]]\^2\)\)\)}, 
            3  \[Pi]/2 < \[Alpha] <= 2  \[Pi], {
              xpos \[Rule] 
                \(lengthSum\ 
                    \@\(\(-fociSeparation\^2\) + 
                        lengthSum\^2\)\)\/\@\(\(-4\)\ fociSeparation\^2 + 
                      4\ lengthSum\^2 + 4\ lengthSum\^2\ Tan[\[Alpha]]\^2\), 
              ypos \[Rule] 
                \(lengthSum\ \@\(\(-fociSeparation\^2\) + lengthSum\^2\)\ 
                    Tan[\[Alpha]]\)\/\@\(\(-4\)\ fociSeparation\^2 + 
                      4\ lengthSum\^2 + 
                      4\ lengthSum\^2\ Tan[\[Alpha]]\^2\)}]]; \)\)], "Input"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Pattern examples", "Section"],

Cell["\<\
This notebook is on the patterns formed by intersecting sets of concentric \
circles\
\>", "Text"],

Cell[BoxData[
    \(\(Show[
      Graphics[{GrayLevel[0.7], Thickness[0.004], 
          Table[Circle[{\(-3\), 0}, r], {r, 1, 10}], 
          Table[Circle[{3, 0}, r], {r, 1, 10}]}], 
      AspectRatio \[Rule] Automatic, ImageSize \[Rule] 500]; \)\)], "Input"],

Cell["\<\
The intersections of the circles are points on ellipses.  Can you spot the \
ellipses?\
\>", "Text"],

Cell["The next plot shows the circles with some of the ellipses.", "Text"],

Cell[BoxData[
    \(\(Show[
      Graphics[{GrayLevel[0.7], Thickness[0.004], 
          Table[Circle[{\(-3\), 0}, r], {r, 1, 10}], 
          Table[Circle[{3, 0}, r], {r, 1, 10}], {Thickness[0.005], Green, 
            Table[Circle[{0, 0}, {i, Sqrt[i\^2 - 3\^2]}], {i, 3 + 1/2, 8, 
                1/2}]}}], AspectRatio \[Rule] Automatic, 
      ImageSize \[Rule] 500]; \)\)], "Input"],

Cell["\<\
The reason that the ellipses are there is to do with how ellipses were \
originally defined.  Start with two fixed points, and two straight lines \
which stretch out from each of the fixed points and meet at their ends.  Vary \
the length of the lines, subject to the sum of the two lengths being a some \
constant value.  Then an ellipse is traced out by the point at the end of the \
lines.\
\>", "Text"],

Cell["\<\
To see how the curve of an ellipse is traced out by the line stretched \
between the fixed points, execute the code in the next cell, and then animate \
the resulting graphics\
\>", "Text"],

Cell[BoxData[
    \(\(\n\(Table[
      Show[Graphics[{{Blue, 
              Line[{{\(-3\), 0}, 
                  alphaSolution[6, 11, N[alphaValue]], {3, 0}}]}, {
              PointSize[0.04], Red, 
              Point[alphaSolution[6, 11, N[alphaValue]]]}, {
              AbsoluteThickness[2], Green, 
              Circle[{0, 0}, {5.5, Sqrt[5.5\^2 - 3\^2]}, {0, alphaValue}]}}], 
        AspectRatio \[Rule] 1, 
        PlotRange \[Rule] {{\(-6\), 6}, {\(-6\), 6}}], {alphaValue, 0, 
        2  \[Pi], \[Pi]/8}]\ ; \)\)\)], "Input"],

Cell["\<\
In the example below, an ellipse formed by a line of length 11 running \
between the two focii via a point on the ellipse is drawn in green.  The blue \
line shows the line of length 11 running from the left focus for a distance \
of 7 to the ellipse.  Then it takes a distance of 4 to the right focus.  \
Therefore the line meets the ellipse where the circle of radius 7 based on \
the left focus, and circle of radius 4 based on the right focus meet.  These \
particular circles are shown in red.\
\>", "Text"],

Cell[BoxData[
    \(\(Show[
      Graphics[{GrayLevel[0.7], Thickness[0.004], 
          Table[Circle[{\(-3\), 0}, i], {i, 9}], {Red, 
            Circle[{\(-3\), 0}, 7]}, 
          Table[Circle[{3, 0}, i], {i, 9}], {Red, Circle[{3, 0}, 4]}, {
            Thickness[0.005], Green, 
            Table[Circle[{0, 0}, {i, Sqrt[i\^2 - 3\^2]}], {i, 5 + 1/2, 
                5 + 1/2, 1/2}]}, {Blue, 
            Line[{{\(-3\), 0}, 
                \(({xpos, ypos} /. 
                    \((upperSolution /. {fociSeparation \[Rule] 6, 
                          lengthSum \[Rule] 11, leftRadius \[Rule] 7. })\))
                  \), {3, 0}}]}}], AspectRatio \[Rule] Automatic, 
      ImageSize \[Rule] 500]; \)\)], "Input"],

Cell["\<\
The above type picture can be animated, to show how this ellipse goes through \
the intersections of the other circles:\
\>", "Text"],

Cell[BoxData[
    \(\(Join[
      Table[Show[
          Graphics[{GrayLevel[0.7], Thickness[0.004], 
              Table[Circle[{\(-3\), 0}, i], {i, 9}], {Red, 
                Circle[{\(-3\), 0}, j]}, 
              Table[Circle[{3, 0}, i], {i, 9}], {Red, 
                Circle[{3, 0}, 11 - j]}, {Thickness[0.005], Green, 
                Table[Circle[{0, 0}, {i, Sqrt[i\^2 - 3\^2]}], {i, 5 + 1/2, 
                    5 + 1/2, 1/2}]}, {Blue, 
                Line[{{\(-3\), 0}, 
                    \(({xpos, ypos} /. 
                        \((upperSolution /. {fociSeparation \[Rule] 6, 
                              lengthSum \[Rule] 11, leftRadius \[Rule] j})\))
                      \), {3, 0}}]}}], AspectRatio \[Rule] Automatic], {j, 8, 
          3, \(-1\)}], 
      Table[Show[
          Graphics[{GrayLevel[0.7], Thickness[0.004], 
              Table[Circle[{\(-3\), 0}, i], {i, 9}], {Red, 
                Circle[{\(-3\), 0}, j]}, 
              Table[Circle[{3, 0}, i], {i, 9}], {Red, 
                Circle[{3, 0}, 11 - j]}, {Thickness[0.005], Green, 
                Table[Circle[{0, 0}, {i, Sqrt[i\^2 - 3\^2]}], {i, 5 + 1/2, 
                    5 + 1/2, 1/2}]}, {Blue, 
                Line[{{\(-3\), 0}, 
                    \(({xpos, ypos} /. 
                        \((lowerSolution /. {fociSeparation \[Rule] 6, 
                              lengthSum \[Rule] 11, leftRadius \[Rule] j})\))
                      \), {3, 0}}]}}], AspectRatio \[Rule] Automatic], {j, 3, 
          8, 1}]]; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Bibliography", "Section"],

Cell["\<\
Brian Bolt \"The Amazing Mathematical Amusement Arcade\" Cambridge University \
Press 1989, page 34 onwards.
This book also contains descriptions of how hyperbolae and cartoids can be \
draw by joining up the intersections of circles.\
\>", "Text"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowSize->{557, 571},
WindowMargins->{{48, Automatic}, {21, Automatic}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 54, 0, 170, "Title"],
Cell[1796, 53, 72, 0, 28, "SmallText"],
Cell[1871, 55, 224, 3, 44, "SmallText"],

Cell[CellGroupData[{
Cell[2120, 62, 33, 0, 59, "Section"],
Cell[2156, 64, 131, 3, 33, "Text"],
Cell[2290, 69, 97, 2, 33, "Text"],
Cell[2390, 73, 70, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[2485, 78, 36, 0, 47, "Subsection"],
Cell[2524, 80, 117, 3, 33, "Text"],
Cell[2644, 85, 61, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[2730, 90, 129, 3, 43, "Subsubsection"],
Cell[2862, 95, 210, 4, 52, "Text"],

Cell[CellGroupData[{
Cell[3097, 103, 279, 5, 93, "Input"],
Cell[3379, 110, 1032, 20, 70, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[4460, 136, 138, 2, 43, "Subsubsection"],
Cell[4601, 140, 222, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[4848, 147, 342, 7, 113, "Input"],
Cell[5193, 156, 1644, 31, 70, "Output"]
}, Open  ]],
Cell[6852, 190, 60, 1, 30, "Input"],
Cell[6915, 193, 761, 15, 140, "Text"],
Cell[7679, 210, 2939, 55, 591, "Input"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[10679, 272, 35, 0, 59, "Section"],
Cell[10717, 274, 108, 3, 33, "Text"],
Cell[10828, 279, 260, 5, 90, "Input"],
Cell[11091, 286, 110, 3, 33, "Text"],
Cell[11204, 291, 74, 0, 33, "Text"],
Cell[11281, 293, 386, 7, 114, "Input"],
Cell[11670, 302, 416, 7, 90, "Text"],
Cell[12089, 311, 199, 4, 52, "Text"],
Cell[12291, 317, 537, 11, 195, "Input"],
Cell[12831, 330, 522, 8, 109, "Text"],
Cell[13356, 340, 720, 14, 216, "Input"],
Cell[14079, 356, 143, 3, 52, "Text"],
Cell[14225, 361, 1534, 29, 401, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15796, 395, 31, 0, 53, "Section"],
Cell[15830, 397, 258, 5, 90, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

