(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     11427,        336]*)
(*NotebookOutlinePosition[     12225,        364]*)
(*  CellTagsIndexPosition[     12181,        360]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Dart Boards", "Title",
  CellLabelAutoDelete->True],

Cell["This notebook is in a file called DartBoards.nb", "SmallText",
  CellLabelAutoDelete->True],

Cell[TextData[StyleBox[
"N.B.  This notebook was originally supplied with the graphics stripped out.  \
To regenerate the graphics, evaluate the entire notebook.",
  FontColor->RGBColor[1, 0, 0]]], "Text",
  CellLabelAutoDelete->True],

Cell["\<\
This notebook describes the use of dart boards in mental arithmetic \
exercises.  Dart boards are useful in giving the whole class mental \
arithmetic exercises.  

Such exercises are currently done in the first ten minutes of a British \
school \"numeracy hour\".\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[CellGroupData[{

Cell["\<\
Initialisation - execute this section before running examples\
\>", "Section",
  CellLabelAutoDelete->True],

Cell["\<\
Execute the following lines before experimenting with any of the examples.\
\>", "Text",
  CellLabelAutoDelete->True],

Cell["\<\
The following line ensures that the DartBoard function is loaded in.\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(\(Needs["\<PrimaryMath`ArithmeticExercises`DartBoards`\>"]\ ; \)\)], 
  "Input",
  CellLabelAutoDelete->True],

Cell["\<\
The following line ensures that colours can be refered to by name.  (e.g. \
RGBColor[1.,0.,0.] can be called Red).\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(\(Needs["\<Graphics`Colors`\>"]\ ; \)\)], "Input",
  CellLabelAutoDelete->True],

Cell["\<\
The following line ensures that RandomArrangement is available, which is a \
function for rearranging a list in random order, and is used to randomise the \
order of the entries on the DartBoard.\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(\(Needs["\<PrimaryMath`Utilities`RandomArrangements`\>"]\ ; \)\)], 
  "Input",
  CellLabelAutoDelete->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Introduction", "Section",
  CellLabelAutoDelete->True],

Cell["\<\
Dart boards are a way of displaying sequences of numbers in an eye catching \
manner.  The examples below show how the DartBoard function can be used to \
generate numeracy exercises.\
\>", "Text",
  CellLabelAutoDelete->True]
}, Open  ]],

Cell[CellGroupData[{

Cell[" 2 times table", "Section",
  CellLabelAutoDelete->True],

Cell["\<\
Here is an example of a rather plain dart board where there are twelve \
sectors and three rings.  In each sector the numbers form a sequence where \
the number in an outer ring is double that in an adjacent inner ring.  This \
forms the basis of practising the 2 times table.   \
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(\(DartBoard[]; \)\)], "Input",
  CellLabelAutoDelete->True],

Cell["\<\
The above example uses the DartBoard function's default option settings. \
\>", "Text",
  CellLabelAutoDelete->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Options[DartBoard]\)], "Input",
  CellLabelAutoDelete->True],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
      \(DartBoardEntriesVisible \[Rule] True\), ",", 
        \(NumberOfRings \[Rule] 3\), ",", \(NumberOfSectors \[Rule] 12\), 
        ",", \(RingWidths \[Rule] 1\), ",", 
        \(SectorFunction \[Rule] \((#1\ 2&)\)\), ",", 
        \(SectorOffset \[Rule] 0\), ",", \(SectorOuterLabels \[Rule] None\), 
        ",", 
        RowBox[{"SectorOuterLabelsMargin", "\[Rule]", 
          StyleBox["0.5`",
            StyleBoxAutoDelete->True,
            PrintPrecision->1]}], ",", 
        \(SectorRingBackground \[Rule] RGBColor[1, 1, 1]\), ",", 
        \(SectorSeeds \[Rule] {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12}\)}], 
      "}"}]], "Output",
  CellLabel->"Out[8]="]
}, Open  ]],

Cell["\<\
 This sets the inner ring's numbers to enumerate the sectors  (SectorSeeds).  \
The default function which determines the numbers in the outer rings is set \
to double the numbers in the adjacent inner rings (SectorFunction).  
....There are also options for changing the number of rings (NumberOfRings), \
number of sectors (NumberOfSectors), widths of the rings (RingWidths), \
deciding if the numbers should be visible (DartBoardEntriesVisible) and \
setting the background to the beds (SectorRingBackground).
....The functions for putting additional labels outside each sector \
(SectorOuterLabels) and (SectorOuterLabelsMargin) are described at the end of \
this notebook.\
\>", "Text",
  CellLabelAutoDelete->True],

Cell["\<\
The starting example can be made much more eye catching by: making the \
picture larger, using larger fonts, and alternating the colours in the \
beds:\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(\(DartBoard[ImageSize \[Rule] 500, 
      TextStyle \[Rule] {FontSize \[Rule] 14}, 
      SectorRingBackground 
        \[Rule] {{Yellow, Pink, Yellow}, {Pink, Yellow, Pink}, 
          CycleEntries}]; \)\)], "Input",
  CellLabelAutoDelete->True],

Cell["\<\
The teacher can create DartBoards with numbers missing, for the pupils to \
fill in.  For example to omit the central ring of numbers in the above \
example:\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(\(DartBoard[ImageSize \[Rule] 500, 
      TextStyle \[Rule] {FontSize \[Rule] 14}, 
      SectorRingBackground 
        \[Rule] {{Yellow, Pink, Yellow}, {Pink, Yellow, Pink}, CycleEntries}, 
      DartBoardEntriesVisible 
        \[Rule] {{True, False, True}, RepeatLastEntry}]; \)\)], "Input",
  CellLabelAutoDelete->True],

Cell["\<\
Missing out the numbers in the outer and inner rings gives the pupils the \
chance to practise doubling and halving:\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(\(DartBoard[ImageSize \[Rule] 500, 
      TextStyle \[Rule] {FontSize \[Rule] 14}, 
      SectorRingBackground 
        \[Rule] {{Yellow, Pink, Yellow}, {Pink, Yellow, Pink}, CycleEntries}, 
      DartBoardEntriesVisible 
        \[Rule] {{False, True, False}, RepeatLastEntry}]; \)\)], "Input",
  CellLabelAutoDelete->True],

Cell["\<\
The numbers don't have to be in consecutive order. They can be rearranged by \
changing the value of the SectorSeeds option.  (This is the option which \
determines the numbers in the innermost ring.)  Here is the last example, but \
with the numbers now put in random order by applying the RandomArrangement \
function to the original list of 12 numbers:\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(\(DartBoard[ImageSize \[Rule] 500, 
      TextStyle \[Rule] {FontSize \[Rule] 14}, 
      SectorRingBackground 
        \[Rule] {{Yellow, Pink, Yellow}, {Pink, Yellow, Pink}, CycleEntries}, 
      DartBoardEntriesVisible 
        \[Rule] {{False, True, False}, RepeatLastEntry}, 
      SectorSeeds \[Rule] RandomArrangement[Range[12]]]; \)\)], "Input",
  CellLabelAutoDelete->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Another example - adding 3", "Section",
  CellLabelAutoDelete->True],

Cell["\<\
Different number patterns can be used.  For example, to create a dart board \
for practising adding 3, change the function SectorFunction so that it \
creates the numbers in the outer rings by adding 3 to the numbers in the \
inner rings:\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(\(DartBoard[ImageSize \[Rule] 500, 
      TextStyle \[Rule] {FontSize \[Rule] 14}, 
      SectorRingBackground 
        \[Rule] {{Yellow, Pink, Yellow}, {Pink, Yellow, Pink}, CycleEntries}, 
      SectorFunction \[Rule] \((\((# + 3)\)&)\)]; \)\)], "Input",
  CellLabelAutoDelete->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Creating a traditional dartboard", "Section"],

Cell["\<\
A traditional dart board has 20 sectors and a both narrow and wide rings.  \
Differing ring widths can be achieved by using the RingWidths option:\
\>", "Text"],

Cell[BoxData[
    \(\(DartBoard[ImageSize \[Rule] 500, 
      DartBoardEntriesVisible \[Rule] False, NumberOfRings \[Rule] 5, 
      RingWidths \[Rule] {0.1, 1, CycleEntries}, 
      TextStyle \[Rule] {FontSize \[Rule] 14}, 
      SectorRingBackground 
        \[Rule] {{Yellow, Pink, Yellow}, {Pink, Yellow, Pink}, 
          CycleEntries}]; \)\)], "Input",
  CellLabel->"In[14]:="],

Cell["\<\
To create the right number of sectors, the NumberOfSectors option is used.  \
They are labelled using SectorOuterLabels\
\>", "Text"],

Cell[BoxData[
    \(\(DartBoard[ImageSize \[Rule] 500, 
      DartBoardEntriesVisible \[Rule] False, NumberOfRings \[Rule] 5, 
      NumberOfSectors \[Rule] 20, RingWidths \[Rule] {0.1, 1, CycleEntries}, 
      TextStyle \[Rule] {FontSize \[Rule] 14}, 
      SectorRingBackground 
        \[Rule] {{Yellow, Pink, Yellow}, {Pink, Yellow, Pink}, CycleEntries}, 
      SectorOuterLabels 
        \[Rule] {1, 18, 4, 13, 6, 10, 15, 2, 17, 3, 19, 7, 16, 8, 11, 14, 9, 
          12, 5, 20}, SectorOuterLabelsMargin \[Rule] 0.3]; \)\)], "Input",
  CellLabel->"In[15]:="],

Cell["\<\
Finally, to put the 20 sector at the top, the sectors need to be moved around \
by 1/2 a sector width.  This is done using the SectorOffset option.\
\>", "Text"],

Cell[BoxData[
    \(\(DartBoard[ImageSize \[Rule] 500, 
      DartBoardEntriesVisible \[Rule] False, NumberOfRings \[Rule] 5, 
      NumberOfSectors \[Rule] 20, RingWidths \[Rule] {0.1, 1, CycleEntries}, 
      TextStyle \[Rule] {FontSize \[Rule] 14}, 
      SectorRingBackground 
        \[Rule] {{Yellow, Pink, Yellow}, {Pink, Yellow, Pink}, CycleEntries}, 
      SectorOuterLabels 
        \[Rule] {1, 18, 4, 13, 6, 10, 15, 2, 17, 3, 19, 7, 16, 8, 11, 14, 9, 
          12, 5, 20}, SectorOuterLabelsMargin \[Rule] 0.3, 
      SectorOffset \[Rule] 1/2]; \)\)], "Input",
  CellLabel->"In[16]:="]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowSize->{561, 535},
WindowMargins->{{144, Automatic}, {Automatic, 20}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
CellLabelAutoDelete->False,
Magnification->1,
StyleDefinitions -> "Default.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 57, 1, 115, "Title"],
Cell[1799, 54, 97, 1, 28, "SmallText"],
Cell[1899, 57, 234, 4, 52, "Text"],
Cell[2136, 63, 317, 8, 90, "Text"],

Cell[CellGroupData[{
Cell[2478, 75, 117, 3, 59, "Section"],
Cell[2598, 80, 127, 3, 33, "Text"],
Cell[2728, 85, 121, 3, 33, "Text"],
Cell[2852, 90, 129, 3, 30, "Input"],
Cell[2984, 95, 167, 4, 52, "Text"],
Cell[3154, 101, 99, 2, 30, "Input"],
Cell[3256, 105, 248, 5, 52, "Text"],
Cell[3507, 112, 127, 3, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3671, 120, 60, 1, 53, "Section"],
Cell[3734, 123, 236, 5, 52, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4007, 133, 62, 1, 53, "Section"],
Cell[4072, 136, 332, 6, 71, "Text"],
Cell[4407, 144, 79, 2, 30, "Input"],
Cell[4489, 148, 126, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[4640, 155, 80, 2, 30, "Input"],
Cell[4723, 159, 717, 16, 124, "Output"]
}, Open  ]],
Cell[5455, 178, 730, 12, 166, "Text"],
Cell[6188, 192, 204, 5, 52, "Text"],
Cell[6395, 199, 266, 6, 70, "Input"],
Cell[6664, 207, 210, 5, 52, "Text"],
Cell[6877, 214, 343, 7, 90, "Input"],
Cell[7223, 223, 169, 4, 52, "Text"],
Cell[7395, 229, 344, 7, 90, "Input"],
Cell[7742, 238, 408, 7, 90, "Text"],
Cell[8153, 247, 401, 8, 110, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8591, 260, 74, 1, 53, "Section"],
Cell[8668, 263, 291, 6, 71, "Text"],
Cell[8962, 271, 305, 6, 90, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9304, 282, 51, 0, 53, "Section"],
Cell[9358, 284, 170, 3, 52, "Text"],
Cell[9531, 289, 383, 8, 90, "Input"],
Cell[9917, 299, 143, 3, 52, "Text"],
Cell[10063, 304, 563, 10, 170, "Input"],
Cell[10629, 316, 171, 3, 52, "Text"],
Cell[10803, 321, 596, 11, 170, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

