(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      7775,        300]*)
(*NotebookOutlinePosition[      8429,        323]*)
(*  CellTagsIndexPosition[      8385,        319]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Digital Roots", "Title"],

Cell["This notebook is in a file called DigitalRoots.nb", "SmallText"],

Cell[TextData[StyleBox[
"N.B.  This notebook was originally supplied with the graphics stripped out \
to save space.  To regenerate the graphics, evaluate the entire notebook.",
  FontColor->RGBColor[1, 0, 0]]], "SmallText"],

Cell["This notebook is about digital roots.", "Text"],

Cell["Execute the next line before running any examples:", "Text"],

Cell[BoxData[
    \(\(Needs["\<PrimaryMath`NumberGames`DigitalRoots`DigitalRoots`\>"]\ ; 
    \)\)], "Input"],

Cell["\<\
A digital root or a (integer) number is given by the sum of the digits of the \
number.
Normally numbers are written in base 10, and so normally a digital root is \
the sum of the digits of the number when written in base 10.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(DigitalRoot[17]\)], "Input"],

Cell[BoxData[
    \(8\)], "Output"]
}, Open  ]],

Cell["\<\
If the digits of add up to more than 9, then the proceedure is repeated on \
the sum until the result is 9 or less.  So the digital root of even a very \
large number is less than 9: \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(DigitalRoot[98867]\)], "Input"],

Cell[BoxData[
    \(2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Use of digital roots in primary education.", "Section"],

Cell["\<\
Digital roots are used as an aid in helping pupils learn their tables.  This \
is done by getting the students to create a multiplication table, and then to \
spot the patterns which occur in the digital root of the multiplication \
table.   Such exercises are recommended for 8 year olds and occasionally to 7 \
year olds.\
\>", "Text"],

Cell["For example consider the 2 times table:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(times2Table\  = Table[i, {i, 2, 40, 2}]\)], "Input"],

Cell[BoxData[
    \({2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36, 
      38, 40}\)], "Output"]
}, Open  ]],

Cell["This can be compared with their digital roots:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Transpose[{\ times2Table\ , \ DigitalRoot[times2Table]\ }] // TableForm
      \)], "Input"],

Cell[BoxData[
    TagBox[GridBox[{
          {"2", "2"},
          {"4", "4"},
          {"6", "6"},
          {"8", "8"},
          {"10", "1"},
          {"12", "3"},
          {"14", "5"},
          {"16", "7"},
          {"18", "9"},
          {"20", "2"},
          {"22", "4"},
          {"24", "6"},
          {"26", "8"},
          {"28", "1"},
          {"30", "3"},
          {"32", "5"},
          {"34", "7"},
          {"36", "9"},
          {"38", "2"},
          {"40", "4"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Digital Root pattern diagrams", "Section"],

Cell["\<\
Digital root patterns are a way of depicting the patterns in the digital \
roots of lists of numbers.   
Here is an example which takes the 2 times table digital roots considered in \
the last section:\
\>", "Text"],

Cell[BoxData[
    \(\(DigitalRootPattern[\ times2Table]\ ; \)\)], "Input"],

Cell["\<\
Here is the template which can be used by the students in drawing the \
pattern.\
\>", "Text"],

Cell[BoxData[
    \(\(DigitalRootPattern[]\ ; \)\)], "Input"],

Cell["\<\
The convention for the position at which the starting number occurs is not \
fixed.  For example in reference [TVM], on page 13 the end number is \
occupying the 12 o'clock position, and the starting number is to the right of \
this.  But on page 26 it is the start number that is occupying the 12 o'clock \
positoin.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Advanced Topic - Bases", "Section"],

Cell["\<\
This section is on expressing numbers in bases other than 10.  At the time of \
writing this topic does not form a part of the British numeracy strategy.  \
However it has been a part of the curriculum in the past, and may be again in \
the furture.  Therefore it has been included.\
\>", "Text"],

Cell["\<\
But numbers can be expressed in other bases.  For example 17 can be expressed \
in base 8:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(BaseForm[17, 8]\)], "Input"],

Cell[BoxData[
    TagBox[
      InterpretationBox[\("21"\_"8"\),
        17,
        Editable->False],
      (BaseForm[ #, 8]&)]], "Output"]
}, Open  ]],

Cell["\<\
...the little subscript 8 denotes that the figure is given in base 8.\
\>", "Text"],

Cell["\<\
Earlier in the notebook the digital root of 17 (base 10) was computed:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(DigitalRoot[17]\)], "Input"],

Cell[BoxData[
    \(8\)], "Output"]
}, Open  ]],

Cell["In base 8 the digital root is different:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(DigitalRoot[17, Base \[Rule] 8]\)], "Input"],

Cell[BoxData[
    \(3\)], "Output"]
}, Open  ]],

Cell["\<\
The answer obtained from DigitalRoot is still given in base 10.  For \
example\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(DigitalRoot[123456789, Base \[Rule] 8]\)], "Input"],

Cell[BoxData[
    \(1\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "...If you want to see this in base 8, use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s BaseForm:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(BaseForm[DigitalRoot[123456789, Base \[Rule] 8], 8]\)], "Input"],

Cell[BoxData[
    TagBox[
      InterpretationBox[\("1"\_"8"\),
        1,
        Editable->False],
      (BaseForm[ #, 8]&)]], "Output"]
}, Open  ]],

Cell["\<\
Everything done above in base 10 can be repeated in other bases using the \
Base option.  This option also works for DigitalRootPattern.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["References", "Section"],

Cell["\<\
[TVM] \"TVM Number Series 2 teachers guide\"  Channel Four Learning\
\>", "Text"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowSize->{504, 527},
WindowMargins->{{42, Automatic}, {Automatic, 5}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 30, 0, 115, "Title"],
Cell[1772, 53, 70, 0, 28, "SmallText"],
Cell[1845, 55, 224, 3, 44, "SmallText"],
Cell[2072, 60, 53, 0, 33, "Text"],
Cell[2128, 62, 66, 0, 33, "Text"],
Cell[2197, 64, 109, 2, 50, "Input"],
Cell[2309, 68, 249, 5, 71, "Text"],

Cell[CellGroupData[{
Cell[2583, 77, 48, 1, 30, "Input"],
Cell[2634, 80, 35, 1, 29, "Output"]
}, Open  ]],
Cell[2684, 84, 207, 4, 52, "Text"],

Cell[CellGroupData[{
Cell[2916, 92, 51, 1, 30, "Input"],
Cell[2970, 95, 35, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3042, 101, 61, 0, 53, "Section"],
Cell[3106, 103, 347, 6, 109, "Text"],
Cell[3456, 111, 55, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[3536, 115, 72, 1, 30, "Input"],
Cell[3611, 118, 117, 2, 48, "Output"]
}, Open  ]],
Cell[3743, 123, 62, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[3830, 127, 111, 2, 70, "Input"],
Cell[3944, 131, 657, 27, 328, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[4650, 164, 48, 0, 53, "Section"],
Cell[4701, 166, 225, 5, 90, "Text"],
Cell[4929, 173, 74, 1, 30, "Input"],
Cell[5006, 176, 104, 3, 52, "Text"],
Cell[5113, 181, 61, 1, 30, "Input"],
Cell[5177, 184, 341, 6, 109, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5555, 195, 41, 0, 53, "Section"],
Cell[5599, 197, 306, 5, 90, "Text"],
Cell[5908, 204, 114, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[6047, 211, 48, 1, 30, "Input"],
Cell[6098, 214, 140, 5, 29, "Output"]
}, Open  ]],
Cell[6253, 222, 93, 2, 33, "Text"],
Cell[6349, 226, 94, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[6468, 232, 48, 1, 30, "Input"],
Cell[6519, 235, 35, 1, 29, "Output"]
}, Open  ]],
Cell[6569, 239, 56, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[6650, 243, 64, 1, 30, "Input"],
Cell[6717, 246, 35, 1, 29, "Output"]
}, Open  ]],
Cell[6767, 250, 102, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[6894, 257, 71, 1, 30, "Input"],
Cell[6968, 260, 35, 1, 29, "Output"]
}, Open  ]],
Cell[7018, 264, 144, 5, 33, "Text"],

Cell[CellGroupData[{
Cell[7187, 273, 84, 1, 50, "Input"],
Cell[7274, 276, 138, 5, 29, "Output"]
}, Open  ]],
Cell[7427, 284, 160, 3, 52, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7624, 292, 29, 0, 53, "Section"],
Cell[7656, 294, 91, 2, 33, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

