(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      4628,        144]*)
(*NotebookOutlinePosition[      5283,        167]*)
(*  CellTagsIndexPosition[      5239,        163]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Enclosed Shapes", "Title"],

Cell["This notebook's file is called EnclosedShapes.nb", "SmallText"],

Cell[TextData[StyleBox[
"N.B.  This notebook was originally supplied with the graphics stripped out \
to save space.  To regenerate the graphics, evaluate the entire notebook.",
  FontColor->RGBColor[1, 0, 0]]], "SmallText"],

Cell["\<\
This notebook describes filling one polygonal shape with another constructed \
by drawing lines between the sides of the original polygonal shape.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Loading the packages required for the examples", "Section"],

Cell[BoxData[
    \(Needs["\<PrimaryMath`EnclosedShapes`EnclosedShapes`\>"]\)], "Input"],

Cell["\<\
It is convenient to use the shape functions specified in the \
TwoDimensionalShapes package:\
\>", "Text"],

Cell[BoxData[
    \(Needs["\<PrimaryMath`Shapes`TwoDimensionalShapes`\>"]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples involving pentagons", "Section"],

Cell["\<\
The following command uses the commands of the TwoDimensionalShapes package.  \
It creates a object called pentagon which contains a line drawing \
representation of a pentagon.\
\>", "Text"],

Cell[BoxData[
    \(\(pentagon\  = \ 
        LineDrawing[
          RegularPolygon[5, VertexStartAngle \[Rule] Pi/2]]\ ;\)\)], "Input"],

Cell[BoxData[
    \(\(Show[Graphics[pentagon], AspectRatio \[Rule] 1];\)\)], "Input"],

Cell["\<\
The next command uses EnclosedShape (from the EnclosedShapes package), to \
create a shape within the pentagon, where the shape is constructed by drawing \
a line from each side to the next side along from it.  The shape is assigned \
to enclosedPentagonShape[1].\
\>", "Text"],

Cell[BoxData[
    \(\(enclosedPentagonShape[1]\  = 
        EnclosedShape[pentagon, EnclosedShapeState \[Rule] Filled, 
          SideJump \[Rule] 1]\ ;\)\)], "Input"],

Cell["\<\
The next command depicts both the pentagon, and the shape drawn within it.\
\>", "Text"],

Cell[BoxData[
    \(\(Show[Graphics[{pentagon, enclosedPentagonShape[1]}], 
        AspectRatio \[Rule] 1];\)\)], "Input"],

Cell["....the enclosed shape is another pentagon.", "Text"],

Cell["\<\
The next command uses EnclosedShape to create another shape within the \
pentagon, where the shape is constructed by drawing a line from each side to \
2nd side along from it.  The shape is assigned to enclosedPentagonShape[2].\
\>", "Text"],

Cell[BoxData[
    \(\(enclosedPentagonShape[2]\  = 
        EnclosedShape[pentagon, EnclosedShapeState \[Rule] Filled, 
          SideJump \[Rule] 2]\ ;\)\)], "Input"],

Cell["\<\
The next command depicts both the pentagon, and the shape drawn within it.\
\>", "Text"],

Cell[BoxData[
    \(\(Show[Graphics[{pentagon, enclosedPentagonShape[2]}], 
        AspectRatio \[Rule] 1];\)\)], "Input"],

Cell["...The enclosed shape is a pentacle.", "Text"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowSize->{446, 579},
WindowMargins->{{90, Automatic}, {Automatic, 41}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 32, 0, 115, "Title"],
Cell[1774, 53, 69, 0, 28, "SmallText"],
Cell[1846, 55, 224, 3, 44, "SmallText"],
Cell[2073, 60, 170, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[2268, 67, 65, 0, 59, "Section"],
Cell[2336, 69, 88, 1, 50, "Input"],
Cell[2427, 72, 116, 3, 52, "Text"],
Cell[2546, 77, 86, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2669, 83, 47, 0, 59, "Section"],
Cell[2719, 85, 201, 4, 71, "Text"],
Cell[2923, 91, 136, 3, 70, "Input"],
Cell[3062, 96, 85, 1, 30, "Input"],
Cell[3150, 99, 287, 5, 90, "Text"],
Cell[3440, 106, 167, 3, 50, "Input"],
Cell[3610, 111, 98, 2, 33, "Text"],
Cell[3711, 115, 122, 2, 50, "Input"],
Cell[3836, 119, 59, 0, 33, "Text"],
Cell[3898, 121, 251, 4, 71, "Text"],
Cell[4152, 127, 167, 3, 50, "Input"],
Cell[4322, 132, 98, 2, 33, "Text"],
Cell[4423, 136, 122, 2, 50, "Input"],
Cell[4548, 140, 52, 0, 33, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

