(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      8259,        224]*)
(*NotebookOutlinePosition[      9010,        250]*)
(*  CellTagsIndexPosition[      8966,        246]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Eratosthenes Sieve", "Title"],

Cell["This notebook is in a file called EratosthenesSieve.nb", "SmallText",
  CellLabelAutoDelete->True],

Cell[TextData[StyleBox[
"N.B.  This notebook was originally supplied with the graphics stripped out.  \
To regenerate the graphics, evaluate the entire notebook.",
  FontColor->RGBColor[1, 0, 0]]], "Text",
  CellLabelAutoDelete->True],

Cell["\<\
This notebook describes how to find prime numbers using the Sieve of \
Eratosthenes.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
Initialisation - execute this section before running the examples\
\>", "Section"],

Cell["\<\
Execute the following lines before experimenting with any of the examples.\
\>", "Text",
  CellLabelAutoDelete->True],

Cell["\<\
The following line ensures that the NumberGrid function is loaded in.\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(Needs["\<PrimaryMath`NumberGrids`NumberGrids`\>"]\)], "Input"],

Cell["\<\
The following line ensures that colours can be refered to by name.  (e.g. \
RGBColor[1.,0.,0.] can be called Red).\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(\(Needs["\<Graphics`Colors`\>"]\ ; \)\)], "Input",
  CellLabelAutoDelete->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Creating the Sieve of Eratosthenes ", "Section"],

Cell["\<\
This example shows how the Sieve of Eratosthenes works on the numbers up to \
100.\
\>", "Text"],

Cell["Start with a clean grid of numbers", "Text"],

Cell[BoxData[
    \(\(NumberGrid[]; \)\)], "Input"],

Cell["\<\
The first number after 1 is 2, which, as it is only divisible by 1 and \
itself, must be prime.  
Now cross out all the numbers that are divisible by the first prime after 1, \
i.e. 2.  In the next grid, this process is represented by greying out the \
even numbers..\
\>", "Text"],

Cell[BoxData[
    \(\(NumberGrid[
      NumberStyles 
        \[Rule] {{{2}, {
              TextStyle 
                \[Rule] {FontWeight \[Rule] "\<Bold\>", FontSize \[Rule] 16}, 
              Background \[Rule] None}}, {Table[i, {i, 0, 100, 2}], 
            Background \[Rule] SlateGray}}]\ ; \)\)], "Input"],

Cell[TextData[{
  "All numbers left between 2 and ",
  Cell[BoxData[
      \(TraditionalForm\`2\[Times]2 = 4\)]],
  " (which have not been greyed out) must be prime.  i.e. 3 is prime.  The \
first number which is a compound of 2 and 3 is 6.  All numbers below that \
which have not been greyed out must be prime.  So 5 is also prime.\nTo \
proceed further, cross out all the numbers in the grid that are divisible by \
3.  In the next grid, this has been done by shading them red."
}], "Text"],

Cell[BoxData[
    \(\(NumberGrid[
      NumberStyles 
        \[Rule] {{{2, 3, 5}, {
              TextStyle 
                \[Rule] {FontWeight \[Rule] "\<Bold\>", FontSize \[Rule] 16}, 
              Background \[Rule] None}}, {Table[i, {i, 0, 100, 2}], 
            Background \[Rule] SlateGray}, {Table[i, {i, 0, 100, 3}], 
            Background \[Rule] Red}}]\ ; \)\)], "Input"],

Cell[TextData[
"All the numbers left below 3\[Times]5=15 that have not been shaded are not \
divisible by any number less than or equal to 3, and therefore must be prime. \
 Therefore 7, 11 and 13 are prime.  \nCrossing out all the numbers that are \
divisible by 5 (shading them blue) gives:"], "Text"],

Cell[BoxData[
    \(\(NumberGrid[
      NumberStyles 
        \[Rule] {{{2, 3, 5, 7, 11, 13}, {
              TextStyle 
                \[Rule] {FontWeight \[Rule] "\<Bold\>", FontSize \[Rule] 16}, 
              Background \[Rule] None}}, {Table[i, {i, 0, 100, 2}], 
            Background \[Rule] SlateGray}, {Table[i, {i, 0, 100, 3}], 
            Background \[Rule] Red}, {Table[i, {i, 0, 100, 5}], 
            Background \[Rule] Blue}}]\ ; \)\)], "Input"],

Cell[TextData[
"All the numbers left unshaded below 5\[Times]7=35 must be prime.  Therefore \
17, 19, 23, 29, and 31 are prime.\nCrossing out all the numbers that are \
divisible by 7 (shading them pink) gives:"], "Text"],

Cell[BoxData[
    \(\(NumberGrid[
      NumberStyles 
        \[Rule] {{{2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31}, {
              TextStyle 
                \[Rule] {FontWeight \[Rule] "\<Bold\>", FontSize \[Rule] 16}, 
              Background \[Rule] None}}, {Table[i, {i, 0, 100, 2}], 
            Background \[Rule] SlateGray}, {Table[i, {i, 0, 100, 3}], 
            Background \[Rule] Red}, {Table[i, {i, 0, 100, 5}], 
            Background \[Rule] Blue}, {Table[i, {i, 0, 100, 7}], 
            Background \[Rule] Pink}}]\ ; \)\)], "Input"],

Cell[TextData[
"All the numbers left unshaded below 7\[Times]11=77 must be prime.  Therefore \
37, 41, 43, 47, 53, 59, 61, 67, 71 and 73 are prime.\nCrossing out all the \
numbers that are divisible by 11 (shading them orange) gives:  "], "Text"],

Cell[BoxData[
    \(\(NumberGrid[
      NumberStyles 
        \[Rule] {{{2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 
              53, 59, 61, 67, 71, 73}, {
              TextStyle 
                \[Rule] {FontWeight \[Rule] "\<Bold\>", FontSize \[Rule] 16}, 
              Background \[Rule] None}}, {Table[i, {i, 0, 100, 2}], 
            Background \[Rule] SlateGray}, {Table[i, {i, 0, 100, 3}], 
            Background \[Rule] Red}, {Table[i, {i, 0, 100, 5}], 
            Background \[Rule] Blue}, {Table[i, {i, 0, 100, 7}], 
            Background \[Rule] Pink}, {Table[i, {i, 0, 100, 11}], 
            Background \[Rule] Orange}}]\ ; \)\)], "Input"],

Cell[TextData[
"All the numbers left unshaded below 11\[Times]13=143 must be prime.  As 143 \
is larger than the arbitrary limit of 100 for the grid, this last sieve is \
sufficient for finding all the primes under 100.  (The last primes found are \
79, 83, 89 and 97.\nThe final grid looks like:"], "Text"],

Cell[BoxData[
    \(\(NumberGrid[
      NumberStyles 
        \[Rule] {{{2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 
              53, 59, 61, 67, 71, 73, 79, 83, 89, 97}, {
              TextStyle 
                \[Rule] {FontWeight \[Rule] "\<Bold\>", FontSize \[Rule] 16}, 
              Background \[Rule] None}}, {Table[i, {i, 0, 100, 2}], 
            Background \[Rule] SlateGray}, {Table[i, {i, 0, 100, 3}], 
            Background \[Rule] Red}, {Table[i, {i, 0, 100, 5}], 
            Background \[Rule] Blue}, {Table[i, {i, 0, 100, 7}], 
            Background \[Rule] Pink}, {Table[i, {i, 0, 100, 11}], 
            Background \[Rule] Orange}}]\ ; \)\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowSize->{561, 621},
WindowMargins->{{Automatic, 87}, {Automatic, 12}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
StyleDefinitions -> "Default.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 35, 0, 115, "Title"],
Cell[1777, 53, 104, 1, 28, "SmallText"],
Cell[1884, 56, 234, 4, 52, "Text"],
Cell[2121, 62, 108, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[2254, 69, 92, 2, 59, "Section"],
Cell[2349, 73, 127, 3, 33, "Text"],
Cell[2479, 78, 122, 3, 33, "Text"],
Cell[2604, 83, 82, 1, 30, "Input"],
Cell[2689, 86, 167, 4, 33, "Text"],
Cell[2859, 92, 99, 2, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[2995, 99, 54, 0, 39, "Section"],
Cell[3052, 101, 106, 3, 33, "Text"],
Cell[3161, 106, 50, 0, 33, "Text"],
Cell[3214, 108, 51, 1, 30, "Input"],
Cell[3268, 111, 291, 6, 71, "Text"],
Cell[3562, 119, 314, 7, 110, "Input"],
Cell[3879, 128, 493, 9, 109, "Text"],
Cell[4375, 139, 385, 8, 90, "Input"],
Cell[4763, 149, 303, 4, 71, "Text"],
Cell[5069, 155, 462, 9, 110, "Input"],
Cell[5534, 166, 221, 3, 52, "Text"],
Cell[5758, 171, 548, 10, 150, "Input"],
Cell[6309, 183, 246, 3, 71, "Text"],
Cell[6558, 188, 672, 12, 170, "Input"],
Cell[7233, 202, 307, 4, 90, "Text"],
Cell[7543, 208, 688, 12, 170, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

