(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      6578,        204]*)
(*NotebookOutlinePosition[      7234,        227]*)
(*  CellTagsIndexPosition[      7190,        223]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Differences between lists of numbers", "Title"],

Cell["This notebook's file is called NumberDifferences.nb", "SmallText"],

Cell[CellGroupData[{

Cell["\<\
Initialisation and function definitions - execute this section before trying \
the examples\
\>", "Section"],

Cell[BoxData[
    \(\(Options[DifferenceList]\  = 
      \ {\ DifferenceFunction\  -> \ Function[{x1, x2}, Abs[x1 - x2]]\ }\ ; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(DifferenceFunction::usage = 
      "\<DifferenceFunction is an option of DifferenceList which specifies \
the function to be used in taking the differences between successive elements \
of the original list.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(DifferenceList::usage = 
      "\<DifferenceList[list_List] returns a list formed by taking the \
differences between successuve elements in the passed list.  The returned \
list contains the same number of elements as the original list, as the \
difference between the first and last elements is also taken.\>"; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(DifferenceList[l_List, opts___Rule] := \n\t
      Module[{diffFunction, i}, \n
        \t\t\ {\ diffFunction\ }\  = \n\t\t\t\ \ 
          \({\ DifferenceFunction\ }\  /. \ {\ opts}\)\  /. \ 
            Options[DifferenceList]\ ; \n\t\t\ \ 
        Append[\n\t\t\t\ \ 
          Table[diffFunction[l[\([i + 1]\)], l[\([i]\)]\ ], {i, Length[l] - 1}
              \n\t\t\ \ \t]\n\t\t\t, \ 
          diffFunction[l[\([1]\)], l[\([Length[l]]\)]\ ]\n\t\t\t]\ \n\t\t]; 
    \)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell["\<\
Suppose you have a list of numbers.  You can generate another list consisting \
of the differences between successive numbers of the first list.   For \
example\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exampleList\  = \ {\ 1\ , 5\ , 7\ , \ 8\ , \ 2\ , \ 4}\)], "Input"],

Cell[BoxData[
    \({1, 5, 7, 8, 2, 4}\)], "Output"]
}, Open  ]],

Cell["\<\
Then you can create another list consisting of the differences of this \
list.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(DifferenceList[exampleList]\)], "Input"],

Cell[BoxData[
    \({4, 2, 1, 6, 2, 3}\)], "Output"]
}, Open  ]],

Cell["\<\
In the above example the last element of the answer list is the difference \
between the last and first numbers of the original list.\
\>", "Text"],

Cell[TextData[{
  "For how long can you keep on doing this, before all the numbers in the \
list become zero?  \nFor the above example, this can be assessed by using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s FixedPointList function:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(FixedPoint[DifferenceList, exampleList, 2]\)], "Input"],

Cell[BoxData[
    \({2, 1, 5, 4, 1, 1}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "If the DifferenceList operation is applied repeatedly, does the returned \
list ever become the same?  One way of trying to test this is by using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s FixedPoint function.\nThe following line applies DifferenceList to \
exampleList repeatedly, until either the result does not change, or 30 \
applications have passed."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(FixedPoint[DifferenceList, exampleList, 30]\)], "Input"],

Cell[BoxData[
    \({1, 0, 1, 1, 0, 1}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "To see all the results generated from repeatedly taking the differences of \
the original list, use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s FixedPointList function.  The following line applies DifferenceList to \
exampleList repeatedy, until either the result does not change, or 30 \
applications have passed.  It returns a list containing the lists it \
calculated."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(FixedPointList[DifferenceList, exampleList, 30]\)], "Input"],

Cell[BoxData[
    \({{1, 5, 7, 8, 2, 4}, {4, 2, 1, 6, 2, 3}, {2, 1, 5, 4, 1, 1}, {1, 4, 1, 
        3, 0, 1}, {3, 3, 2, 3, 1, 0}, {0, 1, 1, 2, 1, 3}, {1, 0, 1, 1, 2, 
        3}, {1, 1, 0, 1, 1, 2}, {0, 1, 1, 0, 1, 1}, {1, 0, 1, 1, 0, 1}, {1, 
        1, 0, 1, 1, 0}, {0, 1, 1, 0, 1, 1}, {1, 0, 1, 1, 0, 1}, {1, 1, 0, 1, 
        1, 0}, {0, 1, 1, 0, 1, 1}, {1, 0, 1, 1, 0, 1}, {1, 1, 0, 1, 1, 0}, {
        0, 1, 1, 0, 1, 1}, {1, 0, 1, 1, 0, 1}, {1, 1, 0, 1, 1, 0}, {0, 1, 1, 
        0, 1, 1}, {1, 0, 1, 1, 0, 1}, {1, 1, 0, 1, 1, 0}, {0, 1, 1, 0, 1, 
        1}, {1, 0, 1, 1, 0, 1}, {1, 1, 0, 1, 1, 0}, {0, 1, 1, 0, 1, 1}, {1, 
        0, 1, 1, 0, 1}, {1, 1, 0, 1, 1, 0}, {0, 1, 1, 0, 1, 1}, {1, 0, 1, 1, 
        0, 1}}\)], "Output"]
}, Open  ]],

Cell["\<\
...You can see that the results cycle between three states after the 9th \
application of DifferenceList.\
\>", "Text"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowSize->{460, 592},
WindowMargins->{{147, Automatic}, {33, Automatic}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 53, 0, 170, "Title"],
Cell[1795, 53, 72, 0, 28, "SmallText"],

Cell[CellGroupData[{
Cell[1892, 57, 118, 3, 86, "Section"],
Cell[2013, 62, 147, 3, 50, "Input"],
Cell[2163, 67, 249, 4, 90, "Input"],
Cell[2415, 73, 353, 6, 130, "Input"],
Cell[2771, 81, 507, 10, 250, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[3315, 96, 31, 0, 39, "Section"],
Cell[3349, 98, 184, 4, 52, "Text"],

Cell[CellGroupData[{
Cell[3558, 106, 87, 1, 30, "Input"],
Cell[3648, 109, 52, 1, 29, "Output"]
}, Open  ]],
Cell[3715, 113, 102, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[3842, 120, 60, 1, 30, "Input"],
Cell[3905, 123, 52, 1, 29, "Output"]
}, Open  ]],
Cell[3972, 127, 157, 3, 52, "Text"],
Cell[4132, 132, 264, 6, 90, "Text"],

Cell[CellGroupData[{
Cell[4421, 142, 75, 1, 30, "Input"],
Cell[4499, 145, 52, 1, 29, "Output"]
}, Open  ]],
Cell[4566, 149, 406, 8, 109, "Text"],

Cell[CellGroupData[{
Cell[4997, 161, 76, 1, 30, "Input"],
Cell[5076, 164, 52, 1, 29, "Output"]
}, Open  ]],
Cell[5143, 168, 420, 9, 90, "Text"],

Cell[CellGroupData[{
Cell[5588, 181, 80, 1, 30, "Input"],
Cell[5671, 184, 735, 10, 314, "Output"]
}, Open  ]],
Cell[6421, 197, 129, 3, 52, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

