(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      7979,        248]*)
(*NotebookOutlinePosition[      8669,        272]*)
(*  CellTagsIndexPosition[      8625,        268]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Number Grids", "Title"],

Cell["This notebook is in a file called NumberGrids.nb", "SmallText",
  CellLabelAutoDelete->True],

Cell[TextData[StyleBox[
"N.B.  This notebook was originally supplied with the graphics stripped out.  \
To regenerate the graphics, evaluate the entire notebook.",
  FontColor->RGBColor[1, 0, 0]]], "Text",
  CellLabelAutoDelete->True],

Cell["\<\
At present this notebook does not include much discussion or examples of how \
number squares can be used in a school class.\
\>", "Text"],

Cell["\<\
This notebook describes how to generate number grids.  It is done with the \
NumberGrid function.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
Initialisation - execute this section before running the examples\
\>", "Section"],

Cell["\<\
Execute the following lines before experimenting with any of the examples.\
\>", "Text",
  CellLabelAutoDelete->True],

Cell["\<\
The following line ensures that the NumberGrid function is loaded in.\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(Needs["\<PrimaryMath`NumberGrids`NumberGrids`\>"]\)], "Input"],

Cell["\<\
The following line ensures that colours can be refered to by name.  (e.g. \
RGBColor[1.,0.,0.] can be called Red).\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(\(Needs["\<Graphics`Colors`\>"]\ ;\)\)], "Input",
  CellLabelAutoDelete->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell["To create a number grid:", "Text"],

Cell[BoxData[
    \(\(NumberGrid[];\)\)], "Input"],

Cell["\<\
...This is the default number grid, which runs from 1 to 100, and is arranged \
so that 1 is in the top left hand corner.\
\>", "Text"],

Cell["\<\
Lots of different number grids are possible.  The content of the number grids \
are controlled by options.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Options[NumberGrid]\)], "Input"],

Cell[BoxData[
    \({NumberGridColumnOrder \[Rule] LeftToRight, 
      NumberGridStartNumber \[Rule] 1, NumberGridEndNumber \[Rule] 100, 
      NumberGridRowOrder \[Rule] TopToBottom, NumberGridWidth \[Rule] 10, 
      NumberStyles \[Rule] {}}\)], "Output"]
}, Open  ]],

Cell["\<\
...All the options except NumberStyles determine what range of numbers the \
number grid will run over, and how the numbers are arranged.  This is covered \
in the next section.  NumberStyles is used for highlighting numbers, and its \
use is covered in a later section.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Using options to shape number grids", "Section"],

Cell["\<\
You can change the start and end numbers.  For example, starting at 8 and \
running to 40:\
\>", "Text"],

Cell[BoxData[
    \(\(NumberGrid[NumberGridStartNumber \[Rule] 8, 
        NumberGridEndNumber \[Rule] 40];\)\)], "Input"],

Cell["\<\
You can vary the way the rows and columns are ordered.  For example, to \
invert the order of both the rows and columns of the default grid\
\>", "Text"],

Cell[BoxData[
    \(\(NumberGrid[NumberGridColumnOrder \[Rule] RightToLeft, 
        NumberGridRowOrder \[Rule] BottomToTop]\ ;\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Highlighting numbers", "Section"],

Cell["\<\
Numbers are highlighted using the NumberStyles option.  In the following \
example the number 10 is shown in red, in bold font and 14pt font on sky blue \
background:\
\>", "Text"],

Cell[BoxData[
    \(\(NumberGrid[NumberGridStartNumber \[Rule] 8, 
        NumberGridEndNumber \[Rule] 40, 
        NumberStyles \[Rule] {{10, {TextStyle \[Rule] {FontFamily \[Rule] \
"\<Helvetica\>", FontColor \[Rule] Red, FontSize \[Rule] 14, 
                    FontWeight \[Rule] "\<Bold\>"}, 
                Background \[Rule] SkyBlue}}}];\)\)], "Input"],

Cell["\<\
Lists of numbers can be highlighted.   In the following example numbers which \
are a multiple of 5 between 1 and 100 are highlighted in Helvetica 12 point \
font with a skyblue background\
\>", "Text"],

Cell[BoxData[
    \(\(NumberGrid[NumberGridStartNumber \[Rule] 1, 
        NumberGridEndNumber \[Rule] 40, 
        NumberStyles \[Rule] {{Table[
                i, {i, 0, 100, 
                  5}], {TextStyle \[Rule] {FontFamily \[Rule] \
"\<Helvetica\>", FontSize \[Rule] 12}, 
                Background \[Rule] SkyBlue}}}];\)\)], "Input"],

Cell["\<\
More than one list of numbers can be highlighted simultaneously.  In the \
following example, as before, numbers which are a multiple of 5 between 1 and \
100 are highlighted with a skyblue background.  Also numbers which are a \
multiple of 4 are emphasised by using 14 point font,  dark green, and bold \
face.\
\>", "Text"],

Cell[BoxData[
    \(\(NumberGrid[NumberGridStartNumber \[Rule] 1, 
        NumberGridEndNumber \[Rule] 40, 
        NumberStyles \[Rule] {{Table[
                i, {i, 0, 100, 
                  5}], {TextStyle \[Rule] {FontFamily \[Rule] \
"\<Helvetica\>", FontSize \[Rule] 12}, Background \[Rule] SkyBlue}}, {Table[
                i, {i, 0, 100, 
                  4}], {TextStyle \[Rule] {FontSize \[Rule] 14, 
                    FontWeight \[Rule] "\<Bold\>"}}}}];\)\)], "Input"],

Cell["\<\
In the above example, for numbers that are a multiple of both 5 and 4, such \
as 20 and 40, the TextStyle definition associated with the multiples of 5 \
took priority, as that was specified first.  To allow the multiples of 4 \
TextStyle definition to take priority, reverse the order of the definitions \
in the NumberStyles option:\
\>", "Text"],

Cell[BoxData[
    \(\(NumberGrid[NumberGridStartNumber \[Rule] 1, 
        NumberGridEndNumber \[Rule] 40, 
        NumberStyles \[Rule] {{Table[
                i, {i, 0, 100, 
                  4}], {TextStyle \[Rule] {FontSize \[Rule] 14, 
                    FontWeight \[Rule] "\<Bold\>"}}}, {Table[
                i, {i, 0, 100, 
                  5}], {TextStyle \[Rule] {FontFamily \[Rule] \
"\<Helvetica\>", FontSize \[Rule] 12}, 
                Background \[Rule] SkyBlue}}}];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Section"],

Cell["\<\
The notebook named \"ErathosthenesSieve.nb\" uses the NumberGrid function to \
demonstrate a method for finding prime numbers.\
\>", "Text"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowSize->{532, 554},
WindowMargins->{{Automatic, 212}, {Automatic, 55}},
StyleDefinitions -> "Default.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 29, 0, 115, "Title"],
Cell[1771, 53, 98, 1, 28, "SmallText"],
Cell[1872, 56, 234, 4, 52, "Text"],
Cell[2109, 62, 148, 3, 52, "Text"],
Cell[2260, 67, 121, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[2406, 74, 92, 2, 86, "Section"],
Cell[2501, 78, 127, 3, 33, "Text"],
Cell[2631, 83, 122, 3, 33, "Text"],
Cell[2756, 88, 82, 1, 30, "Input"],
Cell[2841, 91, 167, 4, 52, "Text"],
Cell[3011, 97, 98, 2, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3146, 104, 31, 0, 59, "Section"],
Cell[3180, 106, 40, 0, 33, "Text"],
Cell[3223, 108, 50, 1, 30, "Input"],
Cell[3276, 111, 145, 3, 52, "Text"],
Cell[3424, 116, 130, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[3579, 123, 52, 1, 30, "Input"],
Cell[3634, 126, 257, 4, 86, "Output"]
}, Open  ]],
Cell[3906, 133, 294, 5, 71, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4237, 143, 54, 0, 59, "Section"],
Cell[4294, 145, 114, 3, 33, "Text"],
Cell[4411, 150, 122, 2, 50, "Input"],
Cell[4536, 154, 163, 3, 52, "Text"],
Cell[4702, 159, 142, 2, 50, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4881, 166, 39, 0, 59, "Section"],
Cell[4923, 168, 190, 4, 52, "Text"],
Cell[5116, 174, 361, 6, 130, "Input"],
Cell[5480, 182, 212, 4, 52, "Text"],
Cell[5695, 188, 344, 7, 110, "Input"],
Cell[6042, 197, 336, 6, 90, "Text"],
Cell[6381, 205, 486, 9, 150, "Input"],
Cell[6870, 216, 358, 6, 90, "Text"],
Cell[7231, 224, 503, 10, 150, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7771, 239, 27, 0, 59, "Section"],
Cell[7801, 241, 150, 3, 52, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

