(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     13470,        371]*)
(*NotebookOutlinePosition[     14158,        395]*)
(*  CellTagsIndexPosition[     14114,        391]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Number Lines", "Title"],

Cell["This notebook is in a file called NumberLines.nb", "SmallText",
  CellLabelAutoDelete->True],

Cell[TextData[StyleBox[
"N.B.  This notebook was originally supplied with the graphics stripped out.  \
To regenerate the graphics, evaluate the entire notebook.",
  FontColor->RGBColor[1, 0, 0]]], "Text",
  CellLabelAutoDelete->True],

Cell["\<\
This notebook describes how to generate number lines.  It is done with the \
NumberLine function.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
Initialisation - execute this section before running the examples\
\>", "Section"],

Cell["\<\
Execute the following lines before experimenting with any of the examples.\
\>", "Text",
  CellLabelAutoDelete->True],

Cell["\<\
The following line ensures that the NumberLine function is loaded in.\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(Needs["\<PrimaryMath`ArithmeticExercises`NumberLines`\>"]\)], "Input"],

Cell["\<\
The following line ensures that colours can be refered to by name.  (e.g. \
RGBColor[1.,0.,0.] can be called Red).\
\>", "Text",
  CellLabelAutoDelete->True],

Cell[BoxData[
    \(\(Needs["\<Graphics`Colors`\>"]\ ;\)\)], "Input",
  CellLabelAutoDelete->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell["The following line draws a number line between -0.5 and 4.5:", "Text"],

Cell[BoxData[
    \(\(NumberLine[\(-1\)/2, 4.5];\)\)], "Input"],

Cell["\<\
....By default, just the integers are put on the number line.  \
\>", "Text"],

Cell["\<\
Note that the above picture is mostly filled with empty space.  This is \
because by default the picture's size is determined by the defaults for a \
Graphics object, which are usually set to AspectRatio\[Rule]1/GoldenRatio, \
and ImageSize\[Rule]Automatic.  (For the FrontEnd, ImageSize\[Rule]Automatic \
usually means that ImageSize\[Rule]288, where 288 means 288 printers points.) \
 Typically we would like the number line to be thin and long.  For most of \
the rest of this tutorial the picture area will be set to an aspect ratio of \
10 horizontal to 1 vertical, and an image size of 300 by 30.  
\tThe Graphics options could be set by altering the default options of \
Graphics using the SetOptions command.  But to avoid changing/loosing the old \
default settings, in this tutorial they are set by passing them as arguments \
of NumberLine.\
\>", "Text"],

Cell[BoxData[
    \(\(NumberLine[\(-1\)/2, 4.5, ImageSize \[Rule] {300, 30}, 
        AspectRatio \[Rule] 0.1\ ];\)\)], "Input"],

Cell["\<\
Lots of different ways of formating and labelling the number line are \
possible.  These are controlled by options:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Options[NumberLine]\)], "Input"],

Cell[BoxData[
    \({NumberLineIntervalColor \[Rule] RGBColor[0, 1, 0], 
      NumberLineIntervalLength \[Rule] None, 
      NumberLineIntervalStart \[Rule] Automatic, 
      NumberLineNumbers \[Rule] Automatic}\)], "Output"]
}, Open  ]],

Cell["\<\
....The NumberLineNumbers option specifies the numbers to be labelled on the \
number line, and is described in the next section.  The other options specify \
how intervals along the number line are to be drawn.  These are explained in \
a later section. \
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Numbers on the number line", "Section"],

Cell["\<\
Various text styles can be applied to the numbers on the number line.  To \
apply a text style to all the numbers, use the TextStyle option.  Here is the \
introductory example with larger bold text in blue:\
\>", "Text"],

Cell[BoxData[
    \(\(NumberLine[\(-1\)/2, 4.5, ImageSize \[Rule] {300, 60}\ , 
        TextStyle -> {FontColor \[Rule] Blue, FontSize -> 14, 
            FontWeight \[Rule] "\<Bold\>"}, AspectRatio \[Rule] 0.1, 
        ImageSize \[Rule] {300, 30}\ ];\)\)], "Input"],

Cell["\<\
To create a number line running from 0 to 3 with the number 2 highlighted, \
pass a list of the text styles to be used as a list with the number 2.\
\>", "Text"],

Cell[BoxData[
    \(\(NumberLine[\(-1\)/2, 4.5, AspectRatio \[Rule] 0.1, 
        ImageSize \[Rule] {300, 30}, 
        NumberLineNumbers \[Rule] {Automatic, {2, {TextStyle \[Rule] \
{FontSize \[Rule] 20, FontColor \[Rule] Red}}}}, 
        TextStyle -> {FontColor \[Rule] Blue, FontSize -> 14, 
            FontWeight \[Rule] "\<Bold\>"}];\)\)], "Input"],

Cell["\<\
More than one number can be highlighted at once with the same format.  In the \
following example, both 2 and 3 are portrayed in large plain red text \
\>", "Text"],

Cell[BoxData[
    \(\(NumberLine[\(-1\)/2, 4.5, AspectRatio \[Rule] 0.1, 
        ImageSize \[Rule] {300, 30}, 
        NumberLineNumbers \[Rule] {Automatic, {{2, 
                3}, {TextStyle \[Rule] {FontSize \[Rule] 20, 
                    FontColor \[Rule] Red}}}}, 
        TextStyle -> {FontColor \[Rule] Blue, FontSize -> 14, 
            FontWeight \[Rule] "\<Bold\>"}];\)\)], "Input"],

Cell["\<\
You can give styles to more than one set of numbers.  In the following \
example, 2,3, and 4 are shown in large italic red text.  Also 1 and 5 are \
listed as to be shown in small bold type brown text.\
\>", "Text"],

Cell[BoxData[
    \(\(NumberLine[0, 6, AspectRatio \[Rule] 0.1, 
        ImageSize \[Rule] {300, 30}, 
        NumberLineNumbers \[Rule] {Automatic, {{2, 3, 
                4}, {TextStyle \[Rule] {FontSize \[Rule] 20, 
                    FontSlant \[Rule] "\<Italic\>", 
                    FontColor \[Rule] Red}}}, {{1, 
                5}, {TextStyle \[Rule] {FontSize \[Rule] 12, 
                    FontColor \[Rule] Brown, 
                    FontWeight \[Rule] "\<Bold\>"}}}}, 
        TextStyle -> {FontColor \[Rule] Blue, FontSize -> 14, 
            FontWeight \[Rule] "\<Bold\>"}];\)\)], "Input"],

Cell["\<\
If a number is given more than one style, then the rule is that the earlier \
styles take priority over the later ones.
The following example is the same as the last, except that 4 is now included \
in both sets of numbers that are to be given distinct styles.\
\>", "Text"],

Cell[BoxData[
    \(\(NumberLine[0, 6, AspectRatio \[Rule] 0.1, 
        ImageSize \[Rule] {300, 30}, 
        NumberLineNumbers \[Rule] {Automatic, {{2, 3, 
                4}, {TextStyle \[Rule] {FontSize \[Rule] 20, 
                    FontSlant \[Rule] "\<Italic\>"}}}, {{1, 4, 
                5}, {TextStyle \[Rule] {FontSize \[Rule] 12, 
                    FontColor \[Rule] Brown, 
                    FontWeight \[Rule] "\<Bold\>"}}}}, 
        TextStyle -> {FontColor \[Rule] Blue, FontSize -> 14, 
            FontWeight \[Rule] "\<Bold\>"}];\)\)], "Input"],

Cell["...4 is shown using the first style specified for it.", "Text"],

Cell["\<\
The the highlighting of numbers can be used to show counting and \
multiplication tables.  For example, to count in 3s from 1 to 21, it is easy \
to use the Table command to create a list of multilples of 3:\
\>", "Text"],

Cell[BoxData[
    \(\(NumberLine[0, 21, AspectRatio \[Rule] 30. /360, 
        NumberLineNumbers \[Rule] {Automatic, {Table[
                i*3, {i, 1, 
                  21/3}], {TextStyle \[Rule] {FontWeight \[Rule] "\<Bold\>", 
                    FontColor \[Rule] Red}}}}, ImageSize \[Rule] {360, 30}, 
        TextStyle \[Rule] {FontColor \[Rule] Blue, 
            FontSize \[Rule] 10}];\)\)], "Input"],

Cell["\<\
...Note that in this example, the ImageSize has been changed to make the \
image longer.  This avoids the numbers running into each other.\
\>", "Text"],

Cell["\<\
It is very easy to drop all the numbers except the counting in 3s.  Just omit \
the Automatic entry in the argument of NumberLineNumbers:\
\>", "Text"],

Cell[BoxData[
    \(\(NumberLine[0, 21, AspectRatio \[Rule] 30. /360, 
        NumberLineNumbers \[Rule] {{Table[
                i*3, {i, 1, 
                  21/3}], {TextStyle \[Rule] {FontWeight \[Rule] "\<Bold\>", 
                    FontColor \[Rule] Red}}}}, ImageSize \[Rule] {360, 30}, 
        TextStyle \[Rule] {FontColor \[Rule] Blue, 
            FontSize \[Rule] 10}];\)\)], "Input"],

Cell["\<\
Also you can drop the multiples of 3.  When the background is white, a very \
easy way of doing this by printing them in white:\
\>", "Text"],

Cell[BoxData[
    \(\(NumberLine[0, 21, AspectRatio \[Rule] 30. /360, 
        NumberLineNumbers \[Rule] {Automatic, {Table[
                i*3, {i, 1, 
                  21/3}], {TextStyle \[Rule] {FontColor \[Rule] White}}}}, 
        ImageSize \[Rule] {360, 30}, 
        TextStyle \[Rule] {FontColor \[Rule] Blue, 
            FontSize \[Rule] 10}];\)\)], "Input"],

Cell["...the children can then fill in the missing numbers.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Number line intervals", "Section"],

Cell["\<\
To mark out a number line in intervals, use the NumberLineIntervalLength \
option.  For example, to create a number line from 0 to 20, marked out in \
intervals of 5:\
\>", "Text"],

Cell[BoxData[
    \(\(NumberLine[0, 20, AspectRatio \[Rule] 0.1, 
        NumberLineIntervalLength \[Rule] 5, 
        ImageSize \[Rule] {300, 30}];\)\)], "Input"],

Cell["The colour of the interval can be changed:", "Text"],

Cell[BoxData[
    \(\(NumberLine[0, 20, AspectRatio \[Rule] 0.1, 
        NumberLineIntervalLength \[Rule] 5, ImageSize \[Rule] {300, 30}, 
        NumberLineIntervalColor \[Rule] SkyBlue];\)\)], "Input"],

Cell["\<\
To demonstrate repeated multiples of numbers, a repeating pattern of coloured \
intervals can be used.  For example, to show multiples of 6 as sets of 2 x 3:\
\
\>", "Text"],

Cell[BoxData[
    \(\(NumberLine[0, 20, AspectRatio \[Rule] 0.1, 
        NumberLineIntervalLength \[Rule] 3, ImageSize \[Rule] {300, 30}, 
        NumberLineIntervalColor \[Rule] {SkyBlue, Green}];\)\)], "Input"],

Cell["And to show 9 as sets of 3 x 3:", "Text"],

Cell[BoxData[
    \(\(NumberLine[0, 21, AspectRatio \[Rule] 0.09, 
        NumberLineIntervalLength \[Rule] 3, ImageSize \[Rule] {30/0.09, 30}, 
        NumberLineIntervalColor \[Rule] {SkyBlue, Green, Pink}];\)\)], "Input"],

Cell["\<\
...Note that in this example the number line has been lengthened to \
30/0.09=333 points.\
\>", "Text"],

Cell["\<\
With the default initial setting of NumberLineIntervalStart \[Rule] \
Automatic, the intervals start at the beginning of the number line.  Often \
this is what is wanted.  However sometimes it does not give the desired \
results.  For example, here is the last but one example, but with the number \
line starting at -2:\
\>", "Text"],

Cell[BoxData[
    \(\(NumberLine[\(-2\), 20, AspectRatio \[Rule] 0.1, 
        NumberLineIntervalLength \[Rule] 3, ImageSize \[Rule] {300, 30}, 
        NumberLineIntervalColor \[Rule] {SkyBlue, Green}];\)\)], "Input"],

Cell["\<\
...The groups of 3 are now being counted starting from -2.  If this is what \
is wanted that is fine.  However it is more likely that you will want to \
start counting from 0:\
\>", "Text"],

Cell[BoxData[
    \(\(NumberLine[\(-2\), 20, AspectRatio \[Rule] 0.1, 
        NumberLineIntervalLength \[Rule] 3, ImageSize \[Rule] {300, 30}, 
        NumberLineIntervalColor \[Rule] {SkyBlue, Green}, 
        NumberLineIntervalStart \[Rule] 0];\)\)], "Input"],

Cell["\<\
If you also wanted to count in 3s over the negative number range, you could \
specify the NumberLineIntervalStart to be at a negative multiple of 3:\
\>", "Text"],

Cell[BoxData[
    \(\(NumberLine[\(-2\), 20, AspectRatio \[Rule] 0.1, 
        NumberLineIntervalLength \[Rule] 3, ImageSize \[Rule] {300, 30}, 
        NumberLineIntervalColor \[Rule] {SkyBlue, Green}, 
        NumberLineIntervalStart \[Rule] \(-3\)];\)\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowSize->{470, 683},
WindowMargins->{{54, Automatic}, {Automatic, 3}},
StyleDefinitions -> "Default.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 29, 0, 115, "Title"],
Cell[1771, 53, 98, 1, 28, "SmallText"],
Cell[1872, 56, 234, 4, 52, "Text"],
Cell[2109, 62, 121, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[2255, 69, 92, 2, 86, "Section"],
Cell[2350, 73, 127, 3, 33, "Text"],
Cell[2480, 78, 122, 3, 33, "Text"],
Cell[2605, 83, 90, 1, 50, "Input"],
Cell[2698, 86, 167, 4, 52, "Text"],
Cell[2868, 92, 98, 2, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3003, 99, 31, 0, 59, "Section"],
Cell[3037, 101, 76, 0, 33, "Text"],
Cell[3116, 103, 63, 1, 30, "Input"],
Cell[3182, 106, 87, 2, 33, "Text"],
Cell[3272, 110, 875, 13, 204, "Text"],
Cell[4150, 125, 128, 2, 50, "Input"],
Cell[4281, 129, 139, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[4445, 136, 52, 1, 30, "Input"],
Cell[4500, 139, 225, 4, 86, "Output"]
}, Open  ]],
Cell[4740, 146, 279, 5, 90, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5056, 156, 45, 0, 59, "Section"],
Cell[5104, 158, 231, 4, 71, "Text"],
Cell[5338, 164, 267, 4, 90, "Input"],
Cell[5608, 170, 171, 3, 52, "Text"],
Cell[5782, 175, 355, 6, 170, "Input"],
Cell[6140, 183, 174, 3, 52, "Text"],
Cell[6317, 188, 396, 7, 170, "Input"],
Cell[6716, 197, 225, 4, 71, "Text"],
Cell[6944, 203, 611, 11, 250, "Input"],
Cell[7558, 216, 284, 5, 90, "Text"],
Cell[7845, 223, 570, 10, 250, "Input"],
Cell[8418, 235, 69, 0, 33, "Text"],
Cell[8490, 237, 231, 4, 71, "Text"],
Cell[8724, 243, 410, 7, 130, "Input"],
Cell[9137, 252, 162, 3, 52, "Text"],
Cell[9302, 257, 161, 3, 52, "Text"],
Cell[9466, 262, 399, 7, 130, "Input"],
Cell[9868, 271, 151, 3, 52, "Text"],
Cell[10022, 276, 369, 7, 130, "Input"],
Cell[10394, 285, 69, 0, 33, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10500, 290, 40, 0, 59, "Section"],
Cell[10543, 292, 190, 4, 52, "Text"],
Cell[10736, 298, 163, 3, 70, "Input"],
Cell[10902, 303, 58, 0, 33, "Text"],
Cell[10963, 305, 204, 3, 70, "Input"],
Cell[11170, 310, 183, 4, 52, "Text"],
Cell[11356, 316, 213, 3, 70, "Input"],
Cell[11572, 321, 47, 0, 33, "Text"],
Cell[11622, 323, 224, 3, 90, "Input"],
Cell[11849, 328, 113, 3, 52, "Text"],
Cell[11965, 333, 344, 6, 90, "Text"],
Cell[12312, 341, 218, 3, 70, "Input"],
Cell[12533, 346, 199, 4, 52, "Text"],
Cell[12735, 352, 262, 4, 90, "Input"],
Cell[13000, 358, 172, 3, 52, "Text"],
Cell[13175, 363, 267, 4, 90, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

