(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     12321,        363]*)
(*NotebookOutlinePosition[     13037,        388]*)
(*  CellTagsIndexPosition[     12993,        384]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Pursuit Plots", "Title"],

Cell["This notebook's file is called PursuitPlots.nb", "SmallText"],

Cell[TextData[StyleBox[
"N.B.  This notebook was originally supplied with the graphics stripped out \
to save space.  To regenerate the graphics, evaluate the entire notebook.",
  FontColor->RGBColor[1, 0, 0]]], "SmallText"],

Cell["\<\
This notebook discusses curves which are the paths of pursuers.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Loading the packages required for the examples", "Section"],

Cell[BoxData[
    \(\(Needs["\<PrimaryMath`PursuitPlots`PursuitPlots`\>"]\ ; \)\)], "Input"],

Cell["\<\
...and the following package is required so that named colours can be used in \
the plots:\
\>", "Text"],

Cell[BoxData[
    \(\(Needs["\<Graphics`Colors`\>"]\ ; \)\)], "Input"],

Cell["\<\
The following package is required for the rotations and translations used in \
constructing the tiling:\
\>", "Text"],

Cell[BoxData[
    \(\(Needs["\<PrimaryMath`Shapes`ShapeTranslations`\>"]\ ; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell["\<\
This notebook is about the patterns that can be created from drawing the \
paths pursuers take when pursuing (usually moving) targets.\
\>", "Text"],

Cell["\<\
Imagine a pursuer, say a dog, which is chasing the hare.  If the dog is none \
too bright, it may choose its course after the hare by always running in the \
direction of the hare (as opposed to the smarter option of running towards \
where the hare is running).  The dog could constantly adjust its course to \
correspond to the direction of the hare.  However for the purposes of these \
demonstrations it will be assumed that the dog corrects its course at regular \
intervals.  \
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["A dog chasing a hare which is travelling in a straight line:", "Section"],

Cell["\<\
For the first demonstration it will be assumed that the hare is at point \
{0,0}, and is running to its hole at {0,1}.   The hare can run at 1m/s.  The \
dog will start at {1,0}, and will check its course at 1/10th second \
intervals.  The dog runs at 2m/s.\
\>", "Text"],

Cell[BoxData[
    \(\(example[1] = 
      PursuitCurve[{{0, 1}, {t, 0}}, {t, 0, 1, 0.05}, 
        PursuerSpeed \[Rule] 1.5]\ ; \)\)], "Input"],

Cell["\<\
The usage message for PursuitCurve explains that the first part of the result \
contains the paths the pursuers (- in this case just one pursuer) takes and \
that the second part of the result contains the vectors that the pursuer(s) \
set there course by.\
\>", "Text"],

Cell["Here is the path the pursuer takes:", "Text"],

Cell[BoxData[
    \(\(Show[Graphics[\(example[1]\)[\([1]\)]], 
      AspectRatio \[Rule] Automatic, Frame \[Rule] True]; \)\)], "Input"],

Cell["\<\
And here are the vectors that the pursuer used to set his course:\
\>", "Text"],

Cell[BoxData[
    \(\(Show[Graphics[\(example[1]\)[\([2]\)]], 
      AspectRatio \[Rule] Automatic, Frame \[Rule] True]; \)\)], "Input"],

Cell["\<\
Combining the two plots, using a thicker line for the pursuit path, and \
colouring the pursuit vectors blue: \
\>", "Text"],

Cell[BoxData[
    \(\(Show[
      Graphics[{{Thickness[0.004], Blue, \(example[1]\)[\([2]\)]}, {
            Thickness[0.01], Black, \(example[1]\)[\([1]\)]}}], 
      AspectRatio \[Rule] Automatic, Frame \[Rule] True]; \)\)], "Input"],

Cell[CellGroupData[{

Cell["Creating an animation", "Subsection"],

Cell["\<\
The following code creates an animation of the scenario
(The red point is the position of the hare)\
\>", "Text"],

Cell[BoxData[
    \(\(Table[
      Show[Graphics[{{Thickness[0.004], Blue, 
              Map[Take[#, i]&, \(example[1]\)[\([2]\)]\ ]}, {Thickness[0.01], 
              Black, Map[Take[#, i]&, \(example[1]\)[\([1]\)], {2}]}, {Red, 
              PointSize[0.03], Point[{0.05*i - 0.05, 0}]}}], 
        AspectRatio \[Rule] Automatic, 
        PlotRange \[Rule] {{\(-0.1\), 1.1}, {\(-0.1\), 1.1}}], {i, 21}]\ ; 
    \)\)], "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Pursuers chasing other pursuers", "Section"],

Cell["\<\
Some very pretty patterns can be achieved when pursuers are set to chase \
other pursuers.  For example, consider three dogs who start at the apexes of \
an equilateral triangle.  Each dog is set to chase the dog to his right.\
\>", "Text"],

Cell[BoxData[
    \(\(example[2] = 
      PursuitCurve[{{{Cos[0. ], Sin[0. ]}, 
            Pursuer[2]}, {{Cos[120. \ Degree], Sin[120. \ Degree]}, 
            Pursuer[3]}, {{Cos[240. \ Degree], Sin[240. \ Degree]}, 
            Pursuer[1]}}, {t, 0, 3.0, 0.1}, PursuerSpeed \[Rule] 0.5]\ ; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(Show[
      Graphics[{{Thickness[0.004], Blue, \(example[2]\)[\([2]\)]}, {
            Thickness[0.01], Black, \(example[2]\)[\([1]\)]}}], 
      AspectRatio \[Rule] Automatic, PlotRange \[Rule] All, 
      ImageSize \[Rule] 500]; \)\)], "Input"],

Cell["\<\
Here is the 4 dog case, with the dogs starting at the corners of a square.\
\>", "Text"],

Cell[BoxData[
    \(\(example[3] = 
      PursuitCurve[{{{Cos[45. \ Degree], Sin[45. \ Degree]}, 
            Pursuer[2]}, {{Cos[135. \ Degree], Sin[135. \ Degree]}, 
            Pursuer[3]}, {{Cos[225. \ Degree], Sin[225. \ Degree]}, 
            Pursuer[4]}, {{Cos[315. \ Degree], Sin[315. \ Degree]}, 
            Pursuer[1]}}, {t, 0, 3.0, 0.1}, PursuerSpeed \[Rule] 0.5]\ ; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(Show[
      Graphics[{{Thickness[0.004], Blue, \(example[3]\)[\([2]\)]}, {
            Thickness[0.01], Black, \(example[3]\)[\([1]\)]}}], 
      AspectRatio \[Rule] Automatic, PlotRange \[Rule] All, 
      ImageSize \[Rule] 500]; \)\)], "Input"],

Cell["\<\
And here is the 5 dog case, the dogs starting on the corners of a regular \
pentagon.\
\>", "Text"],

Cell[TextData[{
  "If you are learning ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", see if you can figure out how the next line generates the correct list \
of pursuer starting points and targets:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pursuerSpecifications\  = \ 
      Table[\ {{Cos[2  \[Pi]\ i/5. ], Sin[2\ \[Pi]\ i/5. ]}, Pursuer[i + 1]}
            \ , \ {i, 5}] /. \ Pursuer[6] \[Rule] Pursuer[1]\)], "Input"],

Cell[BoxData[
    \({{{0.309016994374947451`, 0.951056516295153464`}, 
        Pursuer[2]}, {{\(-0.809016994374947273`\), 0.58778525229247327`}, 
        Pursuer[3]}, {{\(-0.809016994374947273`\), 
          \(-0.587785252292473359`\)}, 
        Pursuer[4]}, {{0.309016994374947229`, \(-0.951056516295153642`\)}, 
        Pursuer[5]}, {{1.`, \(-2.44921270764475451`*^-16\)}, Pursuer[1]}}\)], 
  "Output"]
}, Open  ]],

Cell["...Can you see how to generate the case of hexagons ?", "Text"],

Cell["\<\
For this run, we have to let the pursuers run a little longer:\
\>", "Text"],

Cell[BoxData[
    \(\(example[4] = 
      PursuitCurve[pursuerSpecifications, {t, 0, 3.5, 0.1}, 
        PursuerSpeed \[Rule] 0.5]\ ; \)\)], "Input"],

Cell[BoxData[
    \(\(Show[
      Graphics[{{Thickness[0.004], Blue, \(example[4]\)[\([2]\)]}, {
            Thickness[0.01], Black, \(example[4]\)[\([1]\)]}}], 
      AspectRatio \[Rule] Automatic, PlotRange \[Rule] All, 
      ImageSize \[Rule] 500]; \)\)], "Input"],

Cell["\<\
Here is some generalised code for generating the correct list of pursuer \
starting points and targets for a regular polygon.  As you can see, it is \
rather similar to the pentagon example, except that now, the number of sides \
is a parameter.\
\>", "Text"],

Cell[BoxData[
    \(regularPolygonPursuerSpecifications[n_Integer] := 
      Table[\ {{Cos[2.  \[Pi]\ i/n], Sin[2. \ \[Pi]\ i/n]}, Pursuer[i + 1]}
            \ , \ {i, n}] /. \ Pursuer[n + 1] \[Rule] Pursuer[1]\)], "Input"],

Cell["\<\
And now to use this new function definition in taking the case of 6 dogs \
starting at the corners of a regular hexagon.\
\>", "Text"],

Cell[BoxData[
    \(\(example[5] = 
      PursuitCurve[regularPolygonPursuerSpecifications[6], {t, 0, 4.0, 0.1}, 
        PursuerSpeed \[Rule] 0.5]\ ; \)\)], "Input"],

Cell[BoxData[
    \(\(Show[
      Graphics[{{Thickness[0.004], Blue, \(example[5]\)[\([2]\)]}, {
            Thickness[0.01], Black, \(example[5]\)[\([1]\)]}}], 
      AspectRatio \[Rule] Automatic, PlotRange \[Rule] All, 
      ImageSize \[Rule] 500]; \)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Tiling pursuit curve plots", "Section"],

Cell["\<\
Pursuit plots such as those illustrated in the previous section can be used \
as tiles in a mosaic.
In this section one of the most popular designs, - tiling with triangles, is \
demonstrated.\
\>", "Text"],

Cell["\<\
The starting point is the pursuit plot based on pursuers starting at the \
corners of an equilateral triangle, and chasing each other.\
\>", "Text"],

Cell[BoxData[
    \(\(triangleExample = 
      PursuitCurve[{{{Cos[0. ], Sin[0. ]}, 
            Pursuer[2]}, {{Cos[120. \ Degree], Sin[120. \ Degree]}, 
            Pursuer[3]}, {{Cos[240. \ Degree], Sin[240. \ Degree]}, 
            Pursuer[1]}}, {t, 0, 3.0, 0.1}, PursuerSpeed \[Rule] 0.5]\ ; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(triangleTile\  = 
      Show[Graphics[{{Thickness[0.004], Blue, triangleExample[\([2]\)]}, {
              Thickness[0.01], Black, triangleExample[\([1]\)]}}], 
        AspectRatio \[Rule] Automatic, PlotRange \[Rule] All, 
        ImageSize \[Rule] 500]; \)\)], "Input"],

Cell["\<\
A number of tiles are then assembled.  (MoveShape and TurnShape are functions \
which were loaded in the initialisation section with the ShapeTranslations \
package.)  0.866 is an approximation to the square root of 3/4. - See if you \
can figure out why that is the correct offset in the vertical direction for \
some of the tiles.\
\>", "Text"],

Cell[BoxData[
    \(\(Show[{triangleTile, 
        MoveShape[TurnShape[triangleTile, 180. \ Degree], {1/2, 0.866}], 
        MoveShape[
          TurnShape[triangleTile, \ 180. \ Degree], {1/2, \(-0.866\)}], 
        MoveShape[TurnShape[triangleTile, \ 180. \ Degree], {2, 0}], 
        MoveShape[triangleTile, {1.5, 0.866}], 
        MoveShape[triangleTile, {1.5, \(-0.866\)}]}]\ ; \)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Bibliography", "Section"],

Cell["\<\
Lorraine Mottershead \"Sources of Mathematical Discovery\" Basil Blackwell, \
Oxford, 1978.  
ISBN 0-631-10221-3  pp.158-160.
This book describes pursuit curves, suggests some activities and illustrates \
some more of the patterns that can be produced.\
\>", "Text"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowSize->{557, 481},
WindowMargins->{{96, Automatic}, {Automatic, 5}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 30, 0, 115, "Title"],
Cell[1772, 53, 67, 0, 28, "SmallText"],
Cell[1842, 55, 224, 3, 44, "SmallText"],
Cell[2069, 60, 87, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[2181, 66, 65, 0, 59, "Section"],
Cell[2249, 68, 92, 1, 30, "Input"],
Cell[2344, 71, 114, 3, 33, "Text"],
Cell[2461, 76, 70, 1, 30, "Input"],
Cell[2534, 79, 127, 3, 33, "Text"],
Cell[2664, 84, 91, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2792, 90, 31, 0, 59, "Section"],
Cell[2826, 92, 158, 3, 52, "Text"],
Cell[2987, 97, 506, 8, 109, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[3530, 110, 79, 0, 33, "Section"],
Cell[3612, 112, 281, 5, 71, "Text"],
Cell[3896, 119, 143, 3, 50, "Input"],
Cell[4042, 124, 280, 5, 71, "Text"],
Cell[4325, 131, 51, 0, 33, "Text"],
Cell[4379, 133, 136, 2, 30, "Input"],
Cell[4518, 137, 89, 2, 33, "Text"],
Cell[4610, 141, 136, 2, 30, "Input"],
Cell[4749, 145, 134, 3, 33, "Text"],
Cell[4886, 150, 235, 4, 70, "Input"],

Cell[CellGroupData[{
Cell[5146, 158, 43, 0, 47, "Subsection"],
Cell[5192, 160, 123, 3, 52, "Text"],
Cell[5318, 165, 429, 8, 130, "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[5796, 179, 50, 0, 33, "Section"],
Cell[5849, 181, 250, 4, 71, "Text"],
Cell[6102, 187, 311, 6, 90, "Input"],
Cell[6416, 195, 268, 5, 70, "Input"],
Cell[6687, 202, 98, 2, 33, "Text"],
Cell[6788, 206, 398, 7, 130, "Input"],
Cell[7189, 215, 268, 5, 70, "Input"],
Cell[7460, 222, 109, 3, 33, "Text"],
Cell[7572, 227, 224, 6, 52, "Text"],

Cell[CellGroupData[{
Cell[7821, 237, 199, 3, 70, "Input"],
Cell[8023, 242, 404, 7, 105, "Output"]
}, Open  ]],
Cell[8442, 252, 69, 0, 33, "Text"],
Cell[8514, 254, 86, 2, 33, "Text"],
Cell[8603, 258, 149, 3, 50, "Input"],
Cell[8755, 263, 268, 5, 70, "Input"],
Cell[9026, 270, 269, 5, 71, "Text"],
Cell[9298, 277, 224, 3, 70, "Input"],
Cell[9525, 282, 144, 3, 52, "Text"],
Cell[9672, 287, 166, 3, 50, "Input"],
Cell[9841, 292, 268, 5, 70, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[10146, 302, 45, 0, 33, "Section"],
Cell[10194, 304, 216, 5, 52, "Text"],
Cell[10413, 311, 158, 3, 52, "Text"],
Cell[10574, 316, 316, 6, 90, "Input"],
Cell[10893, 324, 293, 5, 90, "Input"],
Cell[11189, 331, 356, 6, 71, "Text"],
Cell[11548, 339, 398, 7, 130, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[11983, 351, 31, 0, 33, "Section"],
Cell[12017, 353, 276, 6, 90, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

