(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     35155,       1114]*)
(*NotebookOutlinePosition[     35924,       1141]*)
(*  CellTagsIndexPosition[     35880,       1137]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Separated Arithmetic", "Title"],

Cell["This notebook's file is called SeparatedArithmetic.nb", "SmallText"],

Cell["\<\
This notebook describes a series of exercises which emphasise the \
significance of 
digit location when adding, subtracting and multiplying numbers.\
\>", "Text"],

Cell["\<\
Execute the following line before experimenting with any of the examples.\
\>", "Text"],

Cell[BoxData[
    \(\(Needs["\<PrimaryMath`ArithmeticExercises`SeparatedArithmetic`\>"]\ ; 
    \)\)], "Input"],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell["\<\
The arithmetic techniques described below are methods for instilling in the \
pupils the significance that digits in numbers represent units, tens, \
hundreds, etc.\
\>", "Text"],

Cell["\<\
The notebook is arranged as follows.
* The first section is on addition.  Two sorts of addition exercises are \
shown and explained:
\tVertical addition
\tSeparated addition
       A set of graded exercises suitable for 8 year olds are demonstrated \
using vertical addition.
* The second section describes separated subtraction.  The generation of a \
set of teachers answers and pupil's exercises is demonstrated.
* The third section is on partial multiples.  Again the generation of a set \
of exercises and answers is demonstrated.\
\>", "Text"],

Cell["\<\
To allow the creation of exercise, entries in the display of these arithmetic \
methods can be made invisible.
Although suppression of entries (such as omitting intermediate results or \
final answer) is demonstrated for just some of the arithmetic techniques, \
this can be done with all the techniques.  The rule is that any option of \
VerticalAddition, SeparatedAddition, SeparatedSubtraction or PartialMultiples \
with \"Visible\" in the name controls the visibility of part of the output:  
-\tAny of these options can be given the value True or False, which will \
specify whether that part of the output is visible.  
-\tHowever if that part of the output is a list of numbers, then the option \
can be set to a list of True and False (e.g. {True,False,True,True}) which \
specifies which individual entries are visible.  
-\tSimilarly if the part of the output is a table of numbers, the \
corresponding option can be given a table of truth values \
(e.g.{{True,False},{False,False}}) to specify which entries are visible.  

The user is advised to first run through the demonstrations below, then to \
experiment with them, varying the options' values to adjust the entries that \
are visible in the exercises. \
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Addition", "Section"],

Cell[CellGroupData[{

Cell["Vertical Addition", "Subsection"],

Cell["\<\
VerticalAddition generates additions where the numbers are added up \
vertically.  It can operate in a rather traditionally basic form:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(VerticalAddition[24, 235, 421, IntermediateWorking \[Rule] None]\)], 
  "Input"],

Cell[BoxData[
    TagBox[GridBox[{
          {" ", "24"},
          {" ", "235"},
          {"+", "421"},
          {" ", "680"}
          },
        ColumnAlignments->{Right},
        RowLines->{False, False, True, False}],
      DisplayForm]], "Output"]
}, Open  ]],

Cell["\<\
Alternatively, some intermediate calculations can be included.\
\>", "Text"],

Cell["\<\
In the example below, there is some intermediate calculations where  digits \
of the same significance are added, with the most significant digits being \
added first:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(VerticalAddition[24, 235, 421, 
      IntermediateWorking \[Rule] MostSignificantDigitsFirst]\)], "Input"],

Cell[BoxData[
    TagBox[GridBox[{
          {" ", "24"},
          {" ", "235"},
          {"+", "421"},
          {" ", "600"},
          {" ", "70"},
          {" ", "10"},
          {" ", "680"}
          },
        ColumnAlignments->{Right},
        RowLines->{False, False, True, False, False, True, False}],
      DisplayForm]], "Output"]
}, Open  ]],

Cell["\<\
Alternatively the digits could have been added starting with the least \
significant digit first:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(VerticalAddition[24, 235, 421, 
      IntermediateWorking \[Rule] LeastSignificantDigitsFirst]\)], "Input"],

Cell[BoxData[
    TagBox[GridBox[{
          {" ", "24"},
          {" ", "235"},
          {"+", "421"},
          {" ", "10"},
          {" ", "70"},
          {" ", "600"},
          {" ", "680"}
          },
        ColumnAlignments->{Right},
        RowLines->{False, False, True, False, False, True, False}],
      DisplayForm]], "Output"]
}, Open  ]],

Cell["\<\
To generate an exercise for a pupil, some or all of the intermediate workings \
can be omitted.  For example to omit the middle entry in the intermediate \
working of the above problem: \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(VerticalAddition[24, 235, 421, 
      IntermediateWorking \[Rule] LeastSignificantDigitsFirst, 
      IntermediateWorkingVisible \[Rule] {True, False, True}]\)], "Input"],

Cell[BoxData[
    TagBox[GridBox[{
          {" ", "24"},
          {" ", "235"},
          {"+", "421"},
          {" ", "10"},
          {" ", "  "},
          {" ", "600"},
          {" ", "680"}
          },
        ColumnAlignments->{Right},
        RowLines->{False, False, True, False, False, True, False}],
      DisplayForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Graded exercises using VerticalAddition", "Subsubsection"],

Cell[TextData[
"The following cases are examples of a sequence that could form a graded \
series of exercises for years 3 and 4.\nAll these exercises use \
IntermediateWorking \[Rule]MostSignificantDigitsFirst, which is the default \
value of the Option."], "Text"],

Cell["\<\
The following input generates the vertical addition for two random integers, \
where each is below 50.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(VerticalAddition[Random[Integer, {1, 49}], Random[Integer, {1, 49}], 
      IntermediateWorking \[Rule] MostSignificantDigitsFirst, 
      IntermediateWorkingVisible \[Rule] True]\)], "Input"],

Cell[BoxData[
    TagBox[GridBox[{
          {" ", "2"},
          {"+", "34"},
          {" ", "30"},
          {" ", "6"},
          {" ", "36"}
          },
        ColumnAlignments->{Right},
        RowLines->{False, True, False, True, False}],
      DisplayForm]], "Output"]
}, Open  ]],

Cell["\<\
The following input generates the vertical addition for two integers, where \
one lies below 50, and the other lies below 100:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(VerticalAddition[Random[Integer, {1, 49}], Random[Integer, {1, 99}], 
      IntermediateWorking \[Rule] MostSignificantDigitsFirst, 
      IntermediateWorkingVisible \[Rule] True]\)], "Input"],

Cell[BoxData[
    TagBox[GridBox[{
          {" ", "18"},
          {"+", "48"},
          {" ", "50"},
          {" ", "16"},
          {" ", "66"}
          },
        ColumnAlignments->{Right},
        RowLines->{False, True, False, True, False}],
      DisplayForm]], "Output"]
}, Open  ]],

Cell["\<\
The following input generates the vertical addition for two integers, where \
one lies below 50, and is not a multiple of 10, and the other lies below 100, \
and is a multiple of 10\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(VerticalAddition[10*Random[Integer, {0, 4}] + Random[Integer, {1, 9}], 
      10*Random[Integer, {1, 9}], 
      IntermediateWorking \[Rule] MostSignificantDigitsFirst, 
      IntermediateWorkingVisible \[Rule] True]\)], "Input"],

Cell[BoxData[
    TagBox[GridBox[{
          {" ", "27"},
          {"+", "60"},
          {" ", "80"},
          {" ", "7"},
          {" ", "87"}
          },
        ColumnAlignments->{Right},
        RowLines->{False, True, False, True, False}],
      DisplayForm]], "Output"]
}, Open  ]],

Cell["\<\
The following input generates the vertical addition for two integers, where \
both lie below 100, and only one of them is a multiple of 10:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(VerticalAddition[10*Random[Integer, {0, 9}] + Random[Integer, {1, 9}], 
      10*Random[Integer, {1, 9}], 
      IntermediateWorking \[Rule] MostSignificantDigitsFirst, 
      IntermediateWorkingVisible \[Rule] True]\)], "Input"],

Cell[BoxData[
    TagBox[GridBox[{
          {" ", "39"},
          {"+", "10"},
          {" ", "40"},
          {" ", "9"},
          {" ", "49"}
          },
        ColumnAlignments->{Right},
        RowLines->{False, True, False, True, False}],
      DisplayForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Separated Addition", "Subsection"],

Cell["\<\
In educational jargon, SeparatedAddition is an example of \"Expanded informal \
examples without regrouping\".  In essence, SeparatedAddition is where you \
take the some numbers to be added, split them into their units, tens, \
hundreds, thousands and so on, then perform the addition of all the split up \
parts.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(SeparatedAddition[1050, 3345, 4543]\)], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{\(1050 + 3345 + 4543\), \( = \ \), GridBox[{
            {"", "1000", "+", "0", "+", "50", "+", "0"},
            {"+", "3000", "+", "300", "+", "40", "+", "5"},
            {"+", "4000", "+", "500", "+", "40", "+", "3"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], \(\  = \), "8938"}],
      DisplayForm]], "Output"]
}, Open  ]],

Cell["\<\
SeparatedAddition has options which control which of the intermediate numbers \
are visible.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(SeparatedAddition[1050, 3345, 4543, 
      SeparatedNumbersVisible \[Rule] {{False, True, False}, True, False}, 
      FinalAnswerVisible \[Rule] False]\)], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{\(1050 + 3345 + 4543\), \( = \ \), GridBox[{
            {"", "    ", "+", "0", "+", "  ", "+", " "},
            {"+", "3000", "+", "300", "+", "40", "+", "5"},
            {"+", "    ", "+", "   ", "+", "  ", "+", " "}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], \(\  = \), "    "}],
      DisplayForm]], "Output"]
}, Open  ]],

Cell["\<\
The following command creates 5 examples of separated addition, where each \
example involves adding two numbers, and each number is below 1000.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Do[Print[
        SeparatedAddition[Random[Integer, {1, 999}], 
          Random[Integer, {1, 999}]]], {5}]\)], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{\(946 + 666\), \( = \ \), GridBox[{
            {"", "900", "+", "40", "+", "6"},
            {"+", "600", "+", "60", "+", "6"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], \(\  = \), "1612"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{\(205 + 983\), \( = \ \), GridBox[{
            {"", "200", "+", "0", "+", "5"},
            {"+", "900", "+", "80", "+", "3"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], \(\  = \), "1188"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{\(942 + 452\), \( = \ \), GridBox[{
            {"", "900", "+", "40", "+", "2"},
            {"+", "400", "+", "50", "+", "2"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], \(\  = \), "1394"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{\(118 + 660\), \( = \ \), GridBox[{
            {"", "100", "+", "10", "+", "8"},
            {"+", "600", "+", "60", "+", "0"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], \(\  = \), "778"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{\(5 + 54\), \( = \ \), GridBox[{
            {"", "0", "+", "5"},
            {"+", "50", "+", "4"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], \(\  = \), "59"}],
      DisplayForm]], "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Subtraction", "Section"],

Cell["\<\
In educational jargon, SeparatedSubtraction is another example of \"Expanded \
informal examples without regrouping\".  In essence, Separated Subtraction is \
where you take the two numbers to be subtracted from each other, split them \
into their units, tens, hundreds, thousands and so on, then perform the \
subtraction for each of the units, tens, hundreds and so on, and finally add \
up the results.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(SeparatedSubtraction[236, 123]\)], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"236", "-", "123", \( = \ \), GridBox[{
            {" ", "200", "+", "30", "+", "6"},
            {"-", "100", "-", "20", "-", "3"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], \(\  = \), "113"}],
      DisplayForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(SeparatedSubtraction[222, 214]\)], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"222", "-", "214", \( = \ \), GridBox[{
            {" ", "200", "+", "20", "+", "2"},
            {"-", "200", "-", "10", "-", "4"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "200", "+", "20", "+", "12"},
              {"-", "200", "-", "20", "-", "4"}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "8"}],
      DisplayForm]], "Output"]
}, Open  ]],

Cell["\<\
...Note that a second piece of working out is included if any carrying \
between columns is involved. \
\>", "Text"],

Cell[CellGroupData[{

Cell["Generating sets of problems", "Subsection"],

Cell["\<\
Here is how to generate 10 SeparatedSubtraction worked problems.  Each \
problem involves two numbers, each number being under 1000.\
\>", "Text"],

Cell["First of all, the full worked examples are produced.", "Text"],

Cell["\<\
The line below sets the seed for the (pseudo) random number generation to a \
known value, so that generating the same problem set can be repeated:\
\>", "Text"],

Cell[BoxData[
    \(SeedRandom[0]\)], "Input"],

Cell["\<\
In the line below, the a variable is used to pass the result of the first \
random generation into the second random number generation.  This ensures \
that the second randomly generated number is never greater than the first \
randomly generated number.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Do[Print[
        SeparatedSubtraction[a = Random[Integer, {1, 999}], 
          Random[Integer, {1, a}]]], {10}]\)], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"407", "-", "37", \( = \ \), GridBox[{
            {" ", "400", "+", "0", "+", "7"},
            {"-", "0", "-", "30", "-", "7"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "400", "+", "100", "+", "7"},
              {"-", "100", "-", "30", "-", "7"}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "370"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"149", "-", "104", \( = \ \), GridBox[{
            {" ", "100", "+", "40", "+", "9"},
            {"-", "100", "-", "0", "-", "4"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], \(\  = \), "45"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"944", "-", "306", \( = \ \), GridBox[{
            {" ", "900", "+", "40", "+", "4"},
            {"-", "300", "-", "0", "-", "6"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "900", "+", "40", "+", "14"},
              {"-", "300", "-", "10", "-", "6"}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "638"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"279", "-", "102", \( = \ \), GridBox[{
            {" ", "200", "+", "70", "+", "9"},
            {"-", "100", "-", "0", "-", "2"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], \(\  = \), "177"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"837", "-", "487", \( = \ \), GridBox[{
            {" ", "800", "+", "30", "+", "7"},
            {"-", "400", "-", "80", "-", "7"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "800", "+", "130", "+", "7"},
              {"-", "500", "-", "80", "-", "7"}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "350"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"160", "-", "139", \( = \ \), GridBox[{
            {" ", "100", "+", "60", "+", "0"},
            {"-", "100", "-", "30", "-", "9"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "100", "+", "60", "+", "10"},
              {"-", "100", "-", "40", "-", "9"}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "21"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"704", "-", "124", \( = \ \), GridBox[{
            {" ", "700", "+", "0", "+", "4"},
            {"-", "100", "-", "20", "-", "4"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "700", "+", "100", "+", "4"},
              {"-", "200", "-", "20", "-", "4"}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "580"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"639", "-", "615", \( = \ \), GridBox[{
            {" ", "600", "+", "30", "+", "9"},
            {"-", "600", "-", "10", "-", "5"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], \(\  = \), "24"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"637", "-", "470", \( = \ \), GridBox[{
            {" ", "600", "+", "30", "+", "7"},
            {"-", "400", "-", "70", "-", "0"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "600", "+", "130", "+", "7"},
              {"-", "500", "-", "70", "-", "0"}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "167"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"685", "-", "37", \( = \ \), GridBox[{
            {" ", "600", "+", "80", "+", "5"},
            {"-", "0", "-", "30", "-", "7"}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "600", "+", "80", "+", "15"},
              {"-", "0", "-", "40", "-", "7"}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "648"}],
      DisplayForm]], "Print"]
}, Open  ]],

Cell["\<\
Now the examples are repeated, but with spaces for the students to insert the \
intermediate stages and final answer:\
\>", "Text"],

Cell["Reset the random number generator with the same seed value:", "Text"],

Cell[BoxData[
    \(\(SeedRandom[0]\ \ \ \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Do[Print[
        SeparatedSubtraction[a = Random[Integer, {1, 999}], 
          Random[Integer, {1, a}], FinalAnswerVisible \[Rule] False, 
          CarriedStartNumberSeparationVisible \[Rule] False, \n\t\t\t
          CarriedSubtractedNumberSeparationVisible \[Rule] False, 
          StartNumberSeparationVisible \[Rule] False, 
          SubtractedNumberSeparationVisible \[Rule] False]], {10}]\)], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"407", "-", "37", \( = \ \), GridBox[{
            {" ", "   ", "+", " ", "+", " "},
            {"-", " ", "-", "  ", "-", " "}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "   ", "+", "   ", "+", " "},
              {"-", "   ", "-", "  ", "-", " "}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "   "}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"149", "-", "104", \( = \ \), GridBox[{
            {" ", "   ", "+", "  ", "+", " "},
            {"-", "   ", "-", " ", "-", " "}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], \(\  = \), "  "}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"944", "-", "306", \( = \ \), GridBox[{
            {" ", "   ", "+", "  ", "+", " "},
            {"-", "   ", "-", " ", "-", " "}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "   ", "+", "  ", "+", "  "},
              {"-", "   ", "-", "  ", "-", " "}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "   "}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"279", "-", "102", \( = \ \), GridBox[{
            {" ", "   ", "+", "  ", "+", " "},
            {"-", "   ", "-", " ", "-", " "}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], \(\  = \), "   "}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"837", "-", "487", \( = \ \), GridBox[{
            {" ", "   ", "+", "  ", "+", " "},
            {"-", "   ", "-", "  ", "-", " "}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "   ", "+", "   ", "+", " "},
              {"-", "   ", "-", "  ", "-", " "}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "   "}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"160", "-", "139", \( = \ \), GridBox[{
            {" ", "   ", "+", "  ", "+", " "},
            {"-", "   ", "-", "  ", "-", " "}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "   ", "+", "  ", "+", "  "},
              {"-", "   ", "-", "  ", "-", " "}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "  "}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"704", "-", "124", \( = \ \), GridBox[{
            {" ", "   ", "+", " ", "+", " "},
            {"-", "   ", "-", "  ", "-", " "}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "   ", "+", "   ", "+", " "},
              {"-", "   ", "-", "  ", "-", " "}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "   "}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"639", "-", "615", \( = \ \), GridBox[{
            {" ", "   ", "+", "  ", "+", " "},
            {"-", "   ", "-", "  ", "-", " "}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], \(\  = \), "  "}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"637", "-", "470", \( = \ \), GridBox[{
            {" ", "   ", "+", "  ", "+", " "},
            {"-", "   ", "-", "  ", "-", " "}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "   ", "+", "   ", "+", " "},
              {"-", "   ", "-", "  ", "-", " "}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "   "}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"685", "-", "37", \( = \ \), GridBox[{
            {" ", "   ", "+", "  ", "+", " "},
            {"-", " ", "-", "  ", "-", " "}
            },
          ColumnAlignments->{Right},
          RowLines->False,
          ColumnLines->False], 
        RowBox[{\(\  = \), GridBox[{
              {" ", "   ", "+", "  ", "+", "  "},
              {"-", " ", "-", "  ", "-", " "}
              },
            ColumnAlignments->{Right},
            RowLines->False,
            ColumnLines->False]}], \(\  = \), "   "}],
      DisplayForm]], "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Multiplication", "Section"],

Cell["\<\
PartialMultiples depicts a way of multiplying two numbers together.  It \
emphasises the significance of the digits of the numbers in depicting, units, \
10s, 100s, 1000s etc.\
\>", "Text"],

Cell["\<\
PartialMultiples takes two numbers, splits them into their units, tens, \
hundreds, etc.  A table is constructed. The parts of the first number is \
placed the first number in the column on the left hand side of a table, and \
the parts of the second number is placed in the top row of the table.  The \
entries in the body of the table are the multiples of the top row and left \
column entries.  Then the rows are added up into subtotals and these \
subtotals are added up to give the total and answer to the original \
multiplication problem.  Alternatively the columns can be added up into \
subtotals and then these are added up to give the final total.\
\>", "Text"],

Cell["For example to multiple together, 235 and 753:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PartialMultiples[235, 753]\)], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"235", "\[Times]", "753", \( = \ \), GridBox[{
            {" ", "700", "50", "3", \(sub - total\)},
            {"200", "140000", "10000", "600", "150600"},
            {"30", "21000", "1500", "90", "22590"},
            {"5", "3500", "250", "15", "3765"},
            {\(sub - total\), "164500", "11750", "705", "176955"}
            },
          RowLines->{True, False, False, True},
          ColumnLines->{True, False, False, True}], \(\  = \), "176955"}],
      DisplayForm]], "Output"]
}, Open  ]],

Cell["\<\
The following subsections show how to generate problems where students have \
to fill in the blank spaces, and teachers answer sheets for these problems/\
\>", "Text"],

Cell[CellGroupData[{

Cell["Student problems", "Subsection"],

Cell[BoxData[
    \(SeedRandom[2]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Do[
      Print[PartialMultiples[Random[Integer, {1, 99}], 
          Random[Integer, {1, 999}], PartialMultiplesVisible \[Rule] False, 
          FinalAnswerVisible \[Rule] False, 
          BottomRowNumbersVisible \[Rule] True, 
          TopRowNumbersVisible \[Rule] False, 
          LeftColumnNumbersVisible \[Rule] False, 
          RightColumnNumbersVisible \[Rule] False]], {5}]\ ; \)\)], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"74", "\[Times]", "600", \( = \ \), GridBox[{
            {" ", "   ", " ", " ", \(sub - total\)},
            {"  ", "     ", " ", " ", "     "},
            {" ", "    ", " ", " ", "    "},
            {\(sub - total\), "44400", "0", "0", "     "}
            },
          RowLines->{True, False, True},
          ColumnLines->{True, False, False, True}], \(\  = \), "     "}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"61", "\[Times]", "906", \( = \ \), GridBox[{
            {" ", "   ", " ", " ", \(sub - total\)},
            {"  ", "     ", " ", "   ", "     "},
            {" ", "   ", " ", " ", "   "},
            {\(sub - total\), "54900", "0", "366", "     "}
            },
          RowLines->{True, False, True},
          ColumnLines->{True, False, False, True}], \(\  = \), "     "}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"39", "\[Times]", "976", \( = \ \), GridBox[{
            {" ", "   ", "  ", " ", \(sub - total\)},
            {"  ", "     ", "    ", "   ", "     "},
            {" ", "    ", "   ", "  ", "    "},
            {\(sub - total\), "35100", "2730", "234", "     "}
            },
          RowLines->{True, False, True},
          ColumnLines->{True, False, False, True}], \(\  = \), "     "}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"56", "\[Times]", "39", \( = \ \), GridBox[{
            {" ", "  ", " ", \(sub - total\)},
            {"  ", "    ", "   ", "    "},
            {" ", "   ", "  ", "   "},
            {\(sub - total\), "1680", "504", "    "}
            },
          RowLines->{True, False, True},
          ColumnLines->{True, False, True}], \(\  = \), "    "}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"27", "\[Times]", "525", \( = \ \), GridBox[{
            {" ", "   ", "  ", " ", \(sub - total\)},
            {"  ", "     ", "   ", "   ", "     "},
            {" ", "    ", "   ", "  ", "    "},
            {\(sub - total\), "13500", "540", "135", "     "}
            },
          RowLines->{True, False, True},
          ColumnLines->{True, False, False, True}], \(\  = \), "     "}],
      DisplayForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Teacher's answers", "Subsection"],

Cell[BoxData[
    \(SeedRandom[2]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Do[
      Print[PartialMultiples[Random[Integer, {1, 99}], 
          Random[Integer, {1, 999}]]], {5}]\ ; \)\)], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"74", "\[Times]", "600", \( = \ \), GridBox[{
            {" ", "600", "0", "0", \(sub - total\)},
            {"70", "42000", "0", "0", "42000"},
            {"4", "2400", "0", "0", "2400"},
            {\(sub - total\), "44400", "0", "0", "44400"}
            },
          RowLines->{True, False, True},
          ColumnLines->{True, False, False, True}], \(\  = \), "44400"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"61", "\[Times]", "906", \( = \ \), GridBox[{
            {" ", "900", "0", "6", \(sub - total\)},
            {"60", "54000", "0", "360", "54360"},
            {"1", "900", "0", "6", "906"},
            {\(sub - total\), "54900", "0", "366", "55266"}
            },
          RowLines->{True, False, True},
          ColumnLines->{True, False, False, True}], \(\  = \), "55266"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"39", "\[Times]", "976", \( = \ \), GridBox[{
            {" ", "900", "70", "6", \(sub - total\)},
            {"30", "27000", "2100", "180", "29280"},
            {"9", "8100", "630", "54", "8784"},
            {\(sub - total\), "35100", "2730", "234", "38064"}
            },
          RowLines->{True, False, True},
          ColumnLines->{True, False, False, True}], \(\  = \), "38064"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"56", "\[Times]", "39", \( = \ \), GridBox[{
            {" ", "30", "9", \(sub - total\)},
            {"50", "1500", "450", "1950"},
            {"6", "180", "54", "234"},
            {\(sub - total\), "1680", "504", "2184"}
            },
          RowLines->{True, False, True},
          ColumnLines->{True, False, True}], \(\  = \), "2184"}],
      DisplayForm]], "Print"],

Cell[BoxData[
    TagBox[
      RowBox[{"27", "\[Times]", "525", \( = \ \), GridBox[{
            {" ", "500", "20", "5", \(sub - total\)},
            {"20", "10000", "400", "100", "10500"},
            {"7", "3500", "140", "35", "3675"},
            {\(sub - total\), "13500", "540", "135", "14175"}
            },
          RowLines->{True, False, True},
          ColumnLines->{True, False, False, True}], \(\  = \), "14175"}],
      DisplayForm]], "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowSize->{460, 560},
WindowMargins->{{54, Automatic}, {18, Automatic}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
Magnification->1,
StyleDefinitions -> "Default.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 37, 0, 115, "Title"],
Cell[1779, 53, 74, 0, 28, "SmallText"],
Cell[1856, 55, 173, 4, 52, "Text"],
Cell[2032, 61, 97, 2, 33, "Text"],
Cell[2132, 65, 111, 2, 70, "Input"],

Cell[CellGroupData[{
Cell[2268, 71, 31, 0, 59, "Section"],
Cell[2302, 73, 188, 4, 52, "Text"],
Cell[2493, 79, 559, 12, 223, "Text"],
Cell[3055, 93, 1245, 20, 375, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4337, 118, 27, 0, 53, "Section"],

Cell[CellGroupData[{
Cell[4389, 122, 39, 0, 47, "Subsection"],
Cell[4431, 124, 159, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[4615, 131, 100, 2, 50, "Input"],
Cell[4718, 135, 255, 9, 73, "Output"]
}, Open  ]],
Cell[4988, 147, 86, 2, 33, "Text"],
Cell[5077, 151, 191, 4, 52, "Text"],

Cell[CellGroupData[{
Cell[5293, 159, 126, 2, 50, "Input"],
Cell[5422, 163, 345, 12, 123, "Output"]
}, Open  ]],
Cell[5782, 178, 121, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[5928, 185, 127, 2, 70, "Input"],
Cell[6058, 189, 345, 12, 123, "Output"]
}, Open  ]],
Cell[6418, 204, 210, 4, 71, "Text"],

Cell[CellGroupData[{
Cell[6653, 212, 190, 3, 90, "Input"],
Cell[6846, 217, 345, 12, 123, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7228, 234, 64, 0, 43, "Subsubsection"],
Cell[7295, 236, 265, 4, 90, "Text"],
Cell[7563, 242, 126, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[7714, 249, 212, 3, 90, "Input"],
Cell[7929, 254, 279, 10, 104, "Output"]
}, Open  ]],
Cell[8223, 267, 150, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[8398, 274, 212, 3, 90, "Input"],
Cell[8613, 279, 281, 10, 104, "Output"]
}, Open  ]],
Cell[8909, 292, 205, 4, 71, "Text"],

Cell[CellGroupData[{
Cell[9139, 300, 249, 4, 110, "Input"],
Cell[9391, 306, 280, 10, 104, "Output"]
}, Open  ]],
Cell[9686, 319, 163, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[9874, 326, 249, 4, 110, "Input"],
Cell[10126, 332, 280, 10, 104, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[10467, 349, 40, 0, 47, "Subsection"],
Cell[10510, 351, 338, 6, 90, "Text"],

Cell[CellGroupData[{
Cell[10873, 361, 68, 1, 30, "Input"],
Cell[10944, 364, 422, 10, 70, "Output"]
}, Open  ]],
Cell[11381, 377, 116, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[11522, 384, 185, 3, 90, "Input"],
Cell[11710, 389, 422, 10, 70, "Output"]
}, Open  ]],
Cell[12147, 402, 168, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[12340, 409, 140, 3, 90, "Input"],
Cell[12483, 414, 328, 9, 36, "Print"],
Cell[12814, 425, 327, 9, 36, "Print"],
Cell[13144, 436, 328, 9, 36, "Print"],
Cell[13475, 447, 327, 9, 36, "Print"],
Cell[13805, 458, 298, 9, 36, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[14164, 474, 30, 0, 53, "Section"],
Cell[14197, 476, 429, 7, 109, "Text"],

Cell[CellGroupData[{
Cell[14651, 487, 63, 1, 30, "Input"],
Cell[14717, 490, 333, 9, 54, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15087, 504, 63, 1, 30, "Input"],
Cell[15153, 507, 587, 16, 75, "Output"]
}, Open  ]],
Cell[15755, 526, 126, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[15906, 533, 49, 0, 47, "Subsection"],
Cell[15958, 535, 156, 3, 52, "Text"],
Cell[16117, 540, 68, 0, 33, "Text"],
Cell[16188, 542, 171, 3, 52, "Text"],
Cell[16362, 547, 46, 1, 30, "Input"],
Cell[16411, 550, 278, 5, 90, "Text"],

Cell[CellGroupData[{
Cell[16714, 559, 146, 3, 90, "Input"],
Cell[16863, 564, 584, 16, 57, "Print"],
Cell[17450, 582, 330, 9, 36, "Print"],
Cell[17783, 593, 587, 16, 57, "Print"],
Cell[18373, 611, 331, 9, 36, "Print"],
Cell[18707, 622, 588, 16, 57, "Print"],
Cell[19298, 640, 587, 16, 57, "Print"],
Cell[19888, 658, 587, 16, 57, "Print"],
Cell[20478, 676, 331, 9, 36, "Print"],
Cell[20812, 687, 588, 16, 57, "Print"],
Cell[21403, 705, 583, 16, 57, "Print"]
}, Open  ]],
Cell[22001, 724, 141, 3, 52, "Text"],
Cell[22145, 729, 75, 0, 33, "Text"],
Cell[22223, 731, 56, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[22304, 736, 432, 7, 190, "Input"],
Cell[22739, 745, 584, 16, 36, "Print"],
Cell[23326, 763, 330, 9, 36, "Print"],
Cell[23659, 774, 587, 16, 57, "Print"],
Cell[24249, 792, 331, 9, 36, "Print"],
Cell[24583, 803, 588, 16, 57, "Print"],
Cell[25174, 821, 587, 16, 57, "Print"],
Cell[25764, 839, 587, 16, 57, "Print"],
Cell[26354, 857, 331, 9, 36, "Print"],
Cell[26688, 868, 588, 16, 57, "Print"],
Cell[27279, 886, 583, 16, 36, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[27923, 909, 33, 0, 53, "Section"],
Cell[27959, 911, 199, 4, 52, "Text"],
Cell[28161, 917, 682, 10, 166, "Text"],
Cell[28846, 929, 62, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[28933, 933, 59, 1, 30, "Input"],
Cell[28995, 936, 532, 11, 131, "Output"]
}, Open  ]],
Cell[29542, 950, 177, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[29744, 957, 38, 0, 47, "Subsection"],
Cell[29785, 959, 46, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[29856, 964, 427, 8, 190, "Input"],
Cell[30286, 974, 449, 10, 70, "Print"],
Cell[30738, 986, 451, 10, 70, "Print"],
Cell[31192, 998, 463, 10, 95, "Print"],
Cell[31658, 1010, 418, 10, 70, "Print"],
Cell[32079, 1022, 461, 10, 70, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[32589, 1038, 39, 0, 47, "Subsection"],
Cell[32631, 1040, 46, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[32702, 1045, 144, 3, 70, "Input"],
Cell[32849, 1050, 449, 10, 70, "Print"],
Cell[33301, 1062, 451, 10, 95, "Print"],
Cell[33755, 1074, 463, 10, 95, "Print"],
Cell[34221, 1086, 418, 10, 70, "Print"],
Cell[34642, 1098, 461, 10, 95, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

