(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      7826,        253]*)
(*NotebookOutlinePosition[      8481,        276]*)
(*  CellTagsIndexPosition[      8437,        272]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Shapes constructed from strings stretched at regular intervals between \
lines\
\>", "Title"],

Cell["This notebook's file is called StringShapes.nb", "SmallText"],

Cell[TextData[StyleBox[
"N.B.  This notebook was originally supplied with the graphics stripped out \
to save space.  To regenerate the graphics, evaluate the entire notebook.",
  FontColor->RGBColor[1, 0, 0]]], "SmallText"],

Cell["\<\
This notebook shows how to draw pictures of strings stretched at regular \
intervals along lines.  
This can give a teacher a picture of how a construction of stretched strings \
would look, before the teacher or pupils builds one.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Initialisation", "Section"],

Cell["Loads in the StringShapes package:", "Text"],

Cell[BoxData[
    \(\(Needs["\<PrimaryMath`StringShapes`StringShapes`\>"]\ ; \)\)], "Input"],

Cell["Loads in a standard package which defines colours: ", "Text"],

Cell[BoxData[
    \(\(Needs["\<Graphics`Colors`\>"]\ ; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Shapes Strung between straight lines", "Section"],

Cell["\<\
The following command creates a plot of lines stretched at equal intervals \
between two lines:\
\>", "Text"],

Cell[BoxData[
    \(baseLine[1]\  = \ Line[{{0, 0}, {0, 1}}]\ ; \n
    baseLine[2]\  = \ Line[{{0, 1}, {1, 1}}]\ ; \)], "Input"],

Cell[BoxData[
    \(\(strings\  = \ 
      StringShape[baseLine[1], baseLine[2], NumberOfStretchedLines \[Rule] 10]
        \ ; \)\)], "Input"],

Cell[BoxData[
    \(\(Show[
      Graphics[{{RGBColor[1, 0, 0], baseLine[1], baseLine[2]}, {
            RGBColor[0, 1, 0], strings}}], AspectRatio \[Rule] 1, 
      PlotRange \[Rule] All]; \)\)], "Input"],

Cell["The number of lines used can be varied:", "Text"],

Cell[BoxData[
    \(\(strings\  = \ 
      StringShape[baseLine[1], baseLine[2], NumberOfStretchedLines \[Rule] 20]
        \ ; \)\)], "Input"],

Cell[BoxData[
    \(\(Show[
      Graphics[{{RGBColor[1, 0, 0], baseLine[1], baseLine[2]}, {
            RGBColor[0, 1, 0], strings}}], AspectRatio \[Rule] 1, 
      PlotRange \[Rule] All]; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Strings strung between curves", "Section"],

Cell["You can use segmented curves:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(baseLine[1]\  = \ 
      Line[Table[N[{Cos[angle], Sin[angle] - 0.1}], {angle, 0, Pi/2, Pi/18}]]
        \)], "Input"],

Cell[BoxData[
    \(Line[{{1.`, \(-0.1`\)}, {0.984807753012207953`, 0.07364817766693033`}, {
          0.939692620785908516`, 0.242020143325668701`}, {
          0.866025403784438552`, 0.4`}, {0.766044443118977991`, 
          0.542787609686539251`}, {0.64278760968653934`, 
          0.66604444311897808`}, {0.5`, 0.766025403784438552`}, {
          0.342020143325668835`, 0.839692620785908339`}, {
          0.173648177666930418`, 0.884807753012207953`}, {0, 0.9`}}]\)], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(strings\  = \ 
      StringShape[baseLine[1], baseLine[2], NumberOfStretchedLines \[Rule] 20]
        \ \ ; \)\)], "Input"],

Cell[BoxData[
    \(\(Show[
      Graphics[{{RGBColor[1, 0, 0], baseLine[1], baseLine[2]}, {
            RGBColor[0, 1, 0], strings}}], AspectRatio \[Rule] 1, 
      PlotRange \[Rule] All]; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Closed shapes", "Section"],

Cell["\<\
You can also perform these patterns with closed shapes.  For instance take an \
equilateral triangle\
\>", "Text"],

Cell[BoxData[
    \(\(Needs["\<PrimaryMath`Shapes`TwoDimensionalShapes`\>"]\ ; \)\)], 
  "Input"],

Cell["\<\
The following command creates the outline of an equilateral triangle with its \
\"first\" vertex at its top.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(EquilateralTriangle[1]\  = \ 
      LineDrawing[RegularPolygon[3, VertexStartAngle \[Rule] \[Pi]/2]]\)], 
  "Input"],

Cell[BoxData[
    \(Line[{{0, 1.`}, {\(-0.866025403784438552`\), \(-0.5`\)}, {
          0.866025403784438552`, \(-0.5`\)}, {0, 1.`}}]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(Needs["\<PrimaryMath`Shapes`ShapeTranslations`\>"]; \)\)], "Input"],

Cell[TextData[
"The following command creates a new equilateral triangle by rotating \
EquilateralTriangle[1] by 120 degrees  (\[Pi]/3).  This new triangle occupies \
exactly the same space as the first triangle, but its \"first\" vertex is at \
its bottom left hand corner."], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(EquilateralTriangle[2] = 
      TurnShape[EquilateralTriangle[1], 120\ Degree]\)], "Input"],

Cell[BoxData[
    \(Line[{{\(-0.866025403784438552`\), \(-0.5`\)}, {0.866025403784438552`, 
          \(-0.499999999999999644`\)}, {0.`, 0.999999999999999644`}, {
          \(-0.866025403784438552`\), \(-0.5`\)}}]\)], "Output"]
}, Open  ]],

Cell["\<\
Now to stretch the strings between the two triangles.  The first string \
starts near top vertex and extends to near the left hand vertex.   The other \
strings are arranged around the triangle, going anti-clockwise from the first \
string.\
\>", "Text"],

Cell[BoxData[
    \(\(strings\  = \ 
      StringShape[EquilateralTriangle[1], EquilateralTriangle[2], 
        NumberOfStretchedLines \[Rule] 60]\ ; \)\)], "Input"],

Cell[BoxData[
    \(\(Show[
      Graphics[{EquilateralTriangle[1], 
          EquilateralTriangle[2], {Red, strings}}], 
      AspectRatio \[Rule] Automatic]; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["3 Dimensional constructions", "Section"],

Cell["\<\
All the above examples are done using 2 dimensional objects.  However \
StringShape also works with 3-Dimensional lines.  For example:\
\>", "Text"],

Cell[BoxData[
    \(baseLine[1]\  = \ Line[{{0, 0, 0}, {1, 0, 0}}]\ ; \n
    baseLine[2]\  = \ Line[{{0, 0, 1}, {0, 1, 1}}]\ ; \)], "Input"],

Cell[BoxData[
    \(\(strings\  = \ 
      StringShape[baseLine[1], baseLine[2], NumberOfStretchedLines \[Rule] 10]
        \ ; \)\)], "Input"],

Cell[BoxData[
    \(\(Show[Graphics3D[{baseLine[1], baseLine[2], {Red, strings}}], 
      Axes \[Rule] True]\ ; \)\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowSize->{503, 501},
WindowMargins->{{36, Automatic}, {Automatic, 11}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 103, 3, 280, "Title"],
Cell[1845, 56, 67, 0, 28, "SmallText"],
Cell[1915, 58, 224, 3, 44, "SmallText"],
Cell[2142, 63, 255, 5, 90, "Text"],

Cell[CellGroupData[{
Cell[2422, 72, 33, 0, 59, "Section"],
Cell[2458, 74, 50, 0, 33, "Text"],
Cell[2511, 76, 92, 1, 30, "Input"],
Cell[2606, 79, 67, 0, 33, "Text"],
Cell[2676, 81, 70, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2783, 87, 55, 0, 53, "Section"],
Cell[2841, 89, 119, 3, 33, "Text"],
Cell[2963, 94, 128, 2, 50, "Input"],
Cell[3094, 98, 143, 3, 50, "Input"],
Cell[3240, 103, 205, 4, 90, "Input"],
Cell[3448, 109, 55, 0, 33, "Text"],
Cell[3506, 111, 143, 3, 50, "Input"],
Cell[3652, 116, 205, 4, 90, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3894, 125, 48, 0, 53, "Section"],
Cell[3945, 127, 45, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[4015, 131, 138, 3, 50, "Input"],
Cell[4156, 136, 485, 8, 105, "Output"]
}, Open  ]],
Cell[4656, 147, 145, 3, 50, "Input"],
Cell[4804, 152, 205, 4, 90, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5046, 161, 32, 0, 53, "Section"],
Cell[5081, 163, 124, 3, 52, "Text"],
Cell[5208, 168, 97, 2, 30, "Input"],
Cell[5308, 172, 132, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[5465, 179, 136, 3, 50, "Input"],
Cell[5604, 184, 148, 2, 29, "Output"]
}, Open  ]],
Cell[5767, 189, 89, 1, 30, "Input"],
Cell[5859, 192, 285, 4, 71, "Text"],

Cell[CellGroupData[{
Cell[6169, 200, 111, 2, 50, "Input"],
Cell[6283, 204, 227, 3, 48, "Output"]
}, Open  ]],
Cell[6525, 210, 264, 5, 71, "Text"],
Cell[6792, 217, 165, 3, 50, "Input"],
Cell[6960, 222, 175, 4, 70, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7172, 231, 46, 0, 53, "Section"],
Cell[7221, 233, 158, 3, 52, "Text"],
Cell[7382, 238, 140, 2, 50, "Input"],
Cell[7525, 242, 143, 3, 50, "Input"],
Cell[7671, 247, 127, 2, 50, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

