(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      4508,        139]*)
(*NotebookOutlinePosition[      5162,        162]*)
(*  CellTagsIndexPosition[      5118,        158]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["StyleTables", "Title"],

Cell["This notebook's file is called StyleTables.nb", "SmallText"],

Cell["\<\
The functions in StyleTables are written for use in other packages.  When \
using the PrimaryMath package, the user need never directly access \
StyleTableExpand.  Nevertheless, for users who would like to write their own \
programs which use these functions, they are described below. \
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
Initialisation - execute this section before trying the examples\
\>", "Section"],

Cell[BoxData[
    \(Needs["\<PrimaryMath`Utilities`StyleTables`\>"]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["StyleTableExpand", "Section"],

Cell["\<\
StyleTableExpand is a function which rearranges lists of values and styles \
associated with those values into a list of single values, together with all \
the styles associated with those values.\
\>", "Text"],

Cell["Here is a simple example:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(StyleTableExpand[{1, {{2, 4}, FreshGreen}, {4, {RoseRed}}}, 
      StyleTableOrdering \[Rule] Append]\)], "Input"],

Cell[BoxData[
    \({{1, 2, 4}, {{1, {}}, {2, {FreshGreen}}, {4, {FreshGreen, RoseRed}}}}
      \)], "Output"]
}, Open  ]],

Cell["\<\
The list passed as the first argument contains a number of elements.  The \
first element is an object: 1, that has no styles associated with it.  
The second element associates the style Green with the list of objects: 2 and \
4.  The third element associates the style Red with the list of the object 4. \
 The returned result is a list whose first member is the list of objects \
passes to StyleTableExpand, and whose second member is a list of objects and \
their styles.  Each member of this list of objects and styles has as its \
first element an object, and as its second element a list of styles \
associated with the first object.  \
\>", "Text"],

Cell["\<\
The StyleTableOrdering option determines whether the later style definitions \
are put at the front or the rear of the list of style definitions.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(StyleTableExpand[{{{2, 4}, FreshGreen}, {4, {RoseRed}}}, 
      StyleTableOrdering \[Rule] Prepend]\)], "Input"],

Cell[BoxData[
    \({{2, 4}, {{2, {FreshGreen}}, {4, {RoseRed, FreshGreen}}}}\)], "Output"]
}, Open  ]],

Cell["Some default styles can be assigned:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(StyleTableExpand[{1, {{2, 4}, FreshGreen}, {4, {RoseRed}}}, 
      StyleTableOrdering \[Rule] Append, DefaultStyles \[Rule] {Brown}]\)], 
  "Input"],

Cell[BoxData[
    \({{1, 2, 
        4}, {{1, {Brown}}, {2, {Brown, FreshGreen}}, {
          4, {Brown, FreshGreen, RoseRed}}}}\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowSize->{409, 673},
WindowMargins->{{Automatic, 86}, {Automatic, 5}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 28, 0, 115, "Title"],
Cell[1770, 53, 66, 0, 28, "SmallText"],
Cell[1839, 55, 310, 5, 109, "Text"],

Cell[CellGroupData[{
Cell[2174, 64, 91, 2, 86, "Section"],
Cell[2268, 68, 80, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2385, 74, 35, 0, 59, "Section"],
Cell[2423, 76, 220, 4, 71, "Text"],
Cell[2646, 82, 41, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[2712, 86, 134, 2, 70, "Input"],
Cell[2849, 90, 110, 2, 48, "Output"]
}, Open  ]],
Cell[2974, 95, 666, 10, 185, "Text"],
Cell[3643, 107, 169, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[3837, 114, 132, 2, 70, "Input"],
Cell[3972, 118, 91, 1, 48, "Output"]
}, Open  ]],
Cell[4078, 122, 52, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[4155, 126, 168, 3, 90, "Input"],
Cell[4326, 131, 142, 3, 67, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

