(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      5389,        169]*)
(*NotebookOutlinePosition[      6045,        192]*)
(*  CellTagsIndexPosition[      6001,        188]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Triangular Numbers", "Title"],

Cell["This notebook's file is called TriangularNumbers.nb", "SmallText"],

Cell[TextData[StyleBox[
"N.B.  This notebook was originally supplied with the graphics stripped out \
to save space.  To regenerate the graphics, evaluate the entire notebook.",
  FontColor->RGBColor[1, 0, 0]]], "SmallText"],

Cell[CellGroupData[{

Cell["Initialisation - execute this before running the examples", "Section"],

Cell["\<\
The following line loads in a package which allows English names for colours.\
\
\>", "Text"],

Cell[BoxData[
    \(\(Needs["\<Graphics`Colors`\>"]\ ; \)\)], "Input"],

Cell[TextData[{
  "This is the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " version of the formulae which generate triangular numbers:"
}], "Text"],

Cell[BoxData[
    \(\(TriangularNumber[1] = 1; \)\)], "Input"],

Cell[BoxData[
    \(\(TriangularNumber[i_Integer?Positive] := \ 
      \(TriangularNumber[i] = TriangularNumber[i - 1] + i\); \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell["\<\
Triangular numbers are a series of numbers formed by adding numbers in the \
series, 1,2,3 together. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[TriangularNumber[i], {i, 1, 5}]\)], "Input"],

Cell[BoxData[
    \({1, 3, 6, 10, 15}\)], "Output"]
}, Open  ]],

Cell["\<\
They are called triangular numbers as triangle number n is a representation \
of the number of evenly spaced (1 unit apart) dots in a triangle with \
internal angles 90 degree and 45 degrees, and whose shorter sides are of \
length n.  For example triangle number 5 represents:\
\>", "Text"],

Cell[BoxData[
    \(\(Show[
      Graphics[{PointSize[0.05], Red, 
          Table[Point[{x, y}], {x, 1, 5}, {y, 1, x}]}], 
      AspectRatio \[Rule] Automatic, ImageSize \[Rule] 400, 
      Axes \[Rule] True, PlotRange \[Rule] {{0, 5.5}, {0, 5.5}}]; \)\)], 
  "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Approximation to area of triangle", "Section"],

Cell[TextData[
"Triangular numbers are an approximation to the area given by the triangle.  \
For example considering TriangularNumber[5 ]=15, then this number is an \
approximation to the area contained within a 45 degree triangle with sides of \
length 5, whose area is 1/2 base \[Times] height = 1/2 \[Times] 5 \[Times] 5 \
= 12.5."], "Text"],

Cell[BoxData[
    \(\(Show[
      Graphics[{{PointSize[0.05], Red, 
            Table[Point[{x, y}], {x, 1, 5}, {y, 1, x}]}, {Blue, 
            Line[{{0, 0}, {5, 0}, {5, 5}, {0, 0}}]}}], 
      AspectRatio \[Rule] Automatic, ImageSize \[Rule] 400, 
      Axes \[Rule] True, PlotRange \[Rule] {{0, 5.5}, {0, 5.5}}]; \)\)], 
  "Input"],

Cell["\<\
The reason that TriangularNumber[5] is an overestimate is that it represents \
the area of the 15 (area = 1) squares in the following diagram:\
\>", "Text"],

Cell[BoxData[
    \(\(Show[
      Graphics[{{Yellow, 
            Table[Rectangle[{x - 1, y - 1}, {x, y}], {x, 1, 5}, {y, 1, x}]}, {
            PointSize[0.05], Red, 
            Table[Point[{x, y}], {x, 1, 5}, {y, 1, x}]}, {Blue, 
            Line[{{0, 0}, {5, 0}, {5, 5}, {0, 0}}]}}], 
      AspectRatio \[Rule] Automatic, ImageSize \[Rule] 400, 
      Axes \[Rule] True, PlotRange \[Rule] {{0, 5.5}, {0, 5.5}}]; \)\)], 
  "Input"],

Cell[TextData[
"....and 5 of these squares extend outside the area of the triangle by half \
their area.  So the TriangularNumber is 5 \[Times] 1/2 = 2.5 more than the \
area of the triangle."], "Text"],

Cell["\<\
In general, the TriangularNumber[n] is always n/2 more than the area of its \
triangle.\
\>", "Text"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowSize->{460, 592},
WindowMargins->{{226, Automatic}, {50, Automatic}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 35, 0, 115, "Title"],
Cell[1777, 53, 72, 0, 28, "SmallText"],
Cell[1852, 55, 224, 3, 44, "SmallText"],

Cell[CellGroupData[{
Cell[2101, 62, 76, 0, 86, "Section"],
Cell[2180, 64, 103, 3, 33, "Text"],
Cell[2286, 69, 70, 1, 30, "Input"],
Cell[2359, 72, 161, 5, 52, "Text"],
Cell[2523, 79, 62, 1, 30, "Input"],
Cell[2588, 82, 141, 2, 50, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2766, 89, 31, 0, 59, "Section"],
Cell[2800, 91, 125, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[2950, 98, 70, 1, 30, "Input"],
Cell[3023, 101, 51, 1, 29, "Output"]
}, Open  ]],
Cell[3089, 105, 301, 5, 90, "Text"],
Cell[3393, 112, 269, 6, 90, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3699, 123, 52, 0, 53, "Section"],
Cell[3754, 125, 345, 5, 90, "Text"],
Cell[4102, 132, 334, 7, 110, "Input"],
Cell[4439, 141, 166, 3, 52, "Text"],
Cell[4608, 146, 434, 9, 150, "Input"],
Cell[5045, 157, 202, 3, 52, "Text"],
Cell[5250, 162, 111, 3, 33, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

