(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      6874,        241]*)
(*NotebookOutlinePosition[      7530,        264]*)
(*  CellTagsIndexPosition[      7486,        260]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["ValueTables", "Title"],

Cell["This notebook's file is called ValueTables.nb", "SmallText"],

Cell["\<\
The functions in ValueTables are written for use in other packages.  When \
using the PrimaryMath package, the user need never directly access \
ValueTableExpand and VisibleString.  Nevertheless, for users who would like \
to write their own programs which use these functions, they are described \
below. \
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
Initialisation - execute this section before trying the examples\
\>", "Section"],

Cell[BoxData[
    \(Needs["\<PrimaryMath`Utilities`ValueTables`\>"]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["ValueTableExpand", "Section"],

Cell["\<\
ValueTableExpand is a function which corrects and if necessary expands the \
lengths of tables so that they are of the correct length.  This function has \
been written so that the user need only enter a few values of a table, and \
then the function will extrapolate the other values.\
\>", "Text"],

Cell["Here is a simple example:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ValueTableExpand[True, 6]\)], "Input"],

Cell[BoxData[
    \({True, True, True, True, True, True}\)], "Output"]
}, Open  ]],

Cell["\<\
ValueTableExpand takes the argument True, and expands it into a table of 6 \
Trues.\
\>", "Text"],

Cell["ValueTableExpand also contracts tables when required:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ValueTableExpand[{True, True, True, True, True, True}, 4]\)], "Input"],

Cell[BoxData[
    \({True, True, True, True}\)], "Output"]
}, Open  ]],

Cell[TextData[
"Here is a slightly more complicated example (It assumes that \
Options[ValueTableExpand] = {RepetitionMethod\[Rule]RepeatLastEntry})."], 
  "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ValueTableExpand[{True, False}, 6]\)], "Input"],

Cell[BoxData[
    \({True, False, False, False, False, False}\)], "Output"]
}, Open  ]],

Cell["\<\
ValueTableExpand takes the list of values and expands it to have 6 entries.  \
The last 4 entries are formed by repeating the last specified entry.  Some \
times one wants the rest of the table to be constructed by cycling through \
the manually specified values.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ValueTableExpand[{True, False}, 6, 
      RepetitionMethod \[Rule] CycleEntries]\)], "Input"],

Cell[BoxData[
    \({True, False, True, False, True, False}\)], "Output"]
}, Open  ]],

Cell["\<\
This last effect could have also been achieved by giving CycleEntries as the \
last entry to the table of values:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ValueTableExpand[{True, False, CycleEntries}, 6]\)], "Input"],

Cell[BoxData[
    \({True, False, True, False, True, False}\)], "Output"]
}, Open  ]],

Cell["\<\
Why the two ways of specifying how the extra entries are created?  This so \
that for more complicated tables, the user has control over how the \
individual sub-lists are expanded.  For example:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ValueTableExpand[{{1, 2, 3, 4}, {a, b, c, d}, {i, j, k}}, 
      5, {6, 6, 8, 8, 8}]\)], "Input"],

Cell[BoxData[
    \({{1, 2, 3, 4, 4, 4}, {a, b, c, d, d, d}, {i, j, k, k, k, k, k, k}, {i, 
        j, k, k, k, k, k, k}, {i, j, k, k, k, k, k, k}}\)], "Output"]
}, Open  ]],

Cell["\<\
The default method of extending the tables can be adjusted using \
ValueTableExpand's option:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ValueTableExpand[{{1, 2, 3, 4}, {a, b, c, d}, {i, j, k}}, 
      5, {6, 6, 8, 8, 8}, RepetitionMethod \[Rule] CycleEntries]\)], "Input"],

Cell[BoxData[
    \({{1, 2, 3, 4, 1, 2}, {a, b, c, d, a, b}, {i, j, k, i, j, k, i, j}, {1, 
        2, 3, 4, 1, 2, 3, 4}, {a, b, c, d, a, b, c, d}}\)], "Output"]
}, Open  ]],

Cell["\<\
However this sets the method for both the top level list, and its sublists.  \
But suppose you want the second sublist to always be expanded by adding on \
copies of the last entry.  Then you could use:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ValueTableExpand[{{1, 2, 3, 4}, {a, b, c, d, RepeatLastEntry}, {i, j, 
          k}}, 5, {6, 6, 8, 8, 8}, RepetitionMethod \[Rule] CycleEntries]\)], 
  "Input"],

Cell[BoxData[
    \({{1, 2, 3, 4, 1, 2}, {a, b, c, d, d, d}, {i, j, k, i, j, k, i, j}, {1, 
        2, 3, 4, 1, 2, 3, 4}, {a, b, c, d, d, d, d, d}}\)], "Output"]
}, Open  ]],

Cell["\<\
The default repetition method is used for all the lists, except for the \
second one, where the RepeatLastEntry method is used.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["VisibleString", "Section"],

Cell["\<\
VisibleString is a function which either converts an expression into a \
string, or returns a string of spaces equal in length to the string form of \
the expression:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(VisibleString[256, True] // InputForm\)], "Input"],

Cell["\"256\"", "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(VisibleString[256, False] // InputForm\)], "Input"],

Cell["\"   \"", "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowSize->{473, 576},
WindowMargins->{{Automatic, 234}, {41, Automatic}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 28, 0, 115, "Title"],
Cell[1770, 53, 66, 0, 28, "SmallText"],
Cell[1839, 55, 330, 6, 90, "Text"],

Cell[CellGroupData[{
Cell[2194, 65, 91, 2, 86, "Section"],
Cell[2288, 69, 80, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2405, 75, 35, 0, 59, "Section"],
Cell[2443, 77, 309, 5, 90, "Text"],
Cell[2755, 84, 41, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[2821, 88, 58, 1, 30, "Input"],
Cell[2882, 91, 70, 1, 29, "Output"]
}, Open  ]],
Cell[2967, 95, 107, 3, 33, "Text"],
Cell[3077, 100, 69, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[3171, 104, 90, 1, 50, "Input"],
Cell[3264, 107, 58, 1, 29, "Output"]
}, Open  ]],
Cell[3337, 111, 163, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[3525, 118, 67, 1, 30, "Input"],
Cell[3595, 121, 75, 1, 29, "Output"]
}, Open  ]],
Cell[3685, 125, 287, 5, 71, "Text"],

Cell[CellGroupData[{
Cell[3997, 134, 113, 2, 50, "Input"],
Cell[4113, 138, 73, 1, 29, "Output"]
}, Open  ]],
Cell[4201, 142, 137, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[4363, 149, 81, 1, 30, "Input"],
Cell[4447, 152, 73, 1, 29, "Output"]
}, Open  ]],
Cell[4535, 156, 219, 4, 71, "Text"],

Cell[CellGroupData[{
Cell[4779, 164, 117, 2, 50, "Input"],
Cell[4899, 168, 161, 2, 67, "Output"]
}, Open  ]],
Cell[5075, 173, 117, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[5217, 180, 156, 2, 50, "Input"],
Cell[5376, 184, 161, 2, 67, "Output"]
}, Open  ]],
Cell[5552, 189, 226, 4, 71, "Text"],

Cell[CellGroupData[{
Cell[5803, 197, 180, 3, 70, "Input"],
Cell[5986, 202, 161, 2, 67, "Output"]
}, Open  ]],
Cell[6162, 207, 151, 3, 52, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6350, 215, 32, 0, 53, "Section"],
Cell[6385, 217, 190, 4, 52, "Text"],

Cell[CellGroupData[{
Cell[6600, 225, 70, 1, 30, "Input"],
Cell[6673, 228, 25, 0, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6735, 233, 71, 1, 30, "Input"],
Cell[6809, 236, 25, 0, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

